calc.dat <- function(muni.jis, T0=FALSE) {
  if(T0) stopifnot((length(muni.jis) == 1) || (length(muni.jis) == 2))
  lat.h13 <- pmax(0, fin.h13[muni.jis,"sfn"] - fin.h13[muni.jis,"sfr"]) / 100
  lat.h20 <- pmax(0, fin.h13[muni.jis,"sfn"] - fin.h13[muni.jis,"sfn.struct.h13"] + fin.h13[muni.jis,"sfn.struct.h20"] - fin.h13[muni.jis,"sfr"]) / 100
  
  coal.h13 <- colSums(df.h13[muni.jis,])
  coal.h13["density"] <- coal.h13["pop"] / coal.h13["area"] * 100
  coal.num.units <- colSums(num.units[muni.jis,])
  coal.coef.h13 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H13"]])))
  coal.coef.h20 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H20"]])))
  coal.sfn.one <- h13.unit.cost * coal.num.units
  coal.sfn.h13 <- h13.unit.cost * coal.num.units * coal.coef.h13
  coal.sfn.h20 <- h20.unit.cost * coal.num.units * coal.coef.h20

  coal.coef.h13.ninv <- coal.coef.h13
  coal.coef.h13.ninv[investment] <- 1
  coal.sfn.h13.ninv <- h13.unit.cost * coal.num.units * coal.coef.h13.ninv
  
  dat <- vector("numeric")
  dat["pop"] <- sum(df.h13[muni.jis, "pop"])
  dat["income"] <- NA
  dat["sfr"] <- sum(fin.h13[muni.jis, "sfr"]) / 100  #change things into $1k equivalents
  dat["sfn.struct.one"] <- sum(coal.sfn.one) / 100 
  dat["sfn.struct.h13"] <- sum(coal.sfn.h13) / 100
  dat["sfn.struct.h13.ninv"] <- sum(coal.sfn.h13.ninv) / 100
  dat["sfn.struct.h20"] <- sum(coal.sfn.h20) / 100
  dat["sfn.subsidy"] <- sum(fin.h13[muni.jis,"sfn"] - fin.h13[muni.jis,"sfn.exsub"]) / 100
  dat["sfn.idio"] <- sum(fin.h13[muni.jis,"sfn.exsub"] - fin.h13[muni.jis,"sfn.struct.h13"]) / 100

  coal.lat.h13 <- max(0, dat["sfn.struct.h13"] + dat["sfn.subsidy"] + dat["sfn.idio"] - dat["sfr"])
  coal.lat.h20 <- max(0, dat["sfn.struct.h20"] + dat["sfn.subsidy"] + dat["sfn.idio"] - dat["sfr"])

  
  dat["santei.h13"] <- max(0, sum(lat.h13) - coal.lat.h13)
  dat["santei.h20"] <- max(0, sum(lat.h20) - coal.lat.h20)
  dat["tokureisai"] <- tokureisai(df.h13[muni.jis, "pop"])

  cells.tmp <- do.call(rbind, cells[muni.jis])
  med <- get.med(cells.tmp)  #should come from mesh.RData
  coal.avg.dist <- sapply(cells[muni.jis], function(x) weighted.mean(spDistsN1(x[,c("lon", "lat"),drop=FALSE], med, longlat=TRUE), x[,"adj.pop"]))

  if(T0) {
      if(length(muni.jis) == 1) ret <- cbind(rbind(dat), dist=coal.avg.dist)
      if(length(muni.jis) == 2) ret <- cbind(rbind(dat, dat), dist=coal.avg.dist)
  } else {
      tmp <- matrix(dat, nrow=length(muni.jis), ncol=length(dat), byrow=TRUE)
      ret <- cbind(tmp, coal.avg.dist)
      colnames(ret) <- c(names(dat), "dist")
  }
  return(ret)
}

