## echo "Rscript --verbose /home2/egw5/japan_restud/2_gappei.R" | qsub -q fas_devel
## takes less than 1 min on 1 cpu

options(echo=TRUE)
proj.home <- "/home2/egw5/japan_restud/"

library("maptools")
load(paste(proj.home, "shp.RData", sep=""))
jis.by.muni <- row.names(shp@data)
names(jis.by.muni) <- paste(shp@data$pref, shp@data$county, shp@data$muni, sep="、")

tmp <- read.csv(paste(proj.home, "shichouson_henkou.txt", sep=""), fileEncoding="UTF-16", sep="\t", as.is=TRUE, strip.white=TRUE)
tmp[396:397, "実施年月日"] <- tmp[395, "実施年月日"]   #fix up the one gun merger we do care about
tmp[396, "旧市町村名"] <- "石川県鳳至郡穴水町"
tmp[397, "旧市町村名"] <- "石川県鳳至郡門前町"
tmp[396:397, "合併等の内容"] <- "county.merger"
tmp[396, "合併.市.町制施行後の状況"] <- "鳳珠郡穴水町(両郡消滅)"
tmp[397, "合併.市.町制施行後の状況"] <- "鳳珠郡門前町(両郡消滅)"
tmp[tmp[,"旧市町村名"]=="山梨県西八代郡上九一色村（分村：富士ケ嶺・本栖・精進）", "旧市町村名"] <- "山梨県西八代郡上九一色村" #all but 363 out of 1639 people
tmp[tmp[,"旧市町村名"]=="山梨県東八代郡中道町、西八代郡上九一色村（分村：梯・古関）", "旧市町村名"] <- "山梨県東八代郡中道町" #ignore the remainder
tmp[,"旧市町村名"] <- gsub("，", "、", tmp[,"旧市町村名"])  #to be able to use strsplit() later
tmp[,"旧市町村名"] <- gsub("､", "、", tmp[,"旧市町村名"])
tmp[,"旧市町村名"] <- gsub("･", "・", tmp[,"旧市町村名"])
tmp[tmp[,"旧市町村名"]=="埼玉県浦和市・大宮市・与野市", "旧市町村名"] <- "埼玉県浦和市、大宮市、与野市" #wrong separator
tmp[tmp[,"旧市町村名"]=="東京都田無市・保谷市", "旧市町村名"] <- "東京都田無市、保谷市"
tmp[tmp[,"合併.市.町制施行後の状況"]=="霞ケ浦町(カスミガウラマチ)", "合併.市.町制施行後の状況"] <- "霞ヶ浦町(カスミガウラマチ)" #fix ケ
tmp[tmp[,"旧市町村名"]=="奈良県添上郡月ケ瀬村、山辺郡都祁村", "旧市町村名"] <- "奈良県添上郡月ヶ瀬村、山辺郡都祁村"
tmp[tmp[,"旧市町村名"]=="静岡県天竜市、浜北市、周智郡春野町、磐田郡龍山村・佐久間町・水窪町、浜名郡舞阪町・雄踏町、引佐郡細江町・引佐町・三ヶ日町", "旧市町村名"] <- 
  "静岡県天竜市、浜北市、周智郡春野町、磐田郡龍山村・佐久間町・水窪町、浜名郡舞阪町・雄踏町、引佐郡細江町・引佐町・三ケ日町"
tmp[tmp[,"旧市町村名"]=="青森県南津軽郡尾上町・平賀町・碇ケ関村", "旧市町村名"] <- "青森県南津軽郡尾上町・平賀町・碇ヶ関村"
tmp[tmp[,"旧市町村名"]=="岩手県一関市、西磐井郡花泉町、東磐井郡大東町・千厩町・東山町室根村・川崎村", "旧市町村名"] <-
  "岩手県一関市、西磐井郡花泉町、東磐井郡大東町・千厩町・東山町・室根村・川崎村" #missing separator
tmp[tmp[,"ＪＩＳコード"]=="１１１１０", "合併.市.町制施行後の状況"] <- "さいたま市(サイタマシイ)" #delete ku
tmp[tmp[,"ＪＩＳコード"]=="１１１１０", "ＪＩＳコード"] <- "１１１００"
tmp[tmp[,"ＪＩＳコード"]=="２６１０８", "合併.市.町制施行後の状況"] <- "京都市(キョウトシ)"
tmp[tmp[,"ＪＩＳコード"]=="２６１０８", "ＪＩＳコード"] <- "２６１００"
tmp[tmp[,"ＪＩＳコード"]=="３４１０８", "合併.市.町制施行後の状況"] <- "広島市"
tmp[tmp[,"ＪＩＳコード"]=="３４１０８", "ＪＩＳコード"] <- "３４１００"
tmp[tmp[,"ＪＩＳコード"]=="２２１０３", "合併.市.町制施行後の状況"] <- "静岡市（シズオカシ)"
tmp[tmp[,"ＪＩＳコード"]=="２２１０３", "ＪＩＳコード"] <- "２２１００"


tmp2 <- tmp[tmp[,1] != "",]  #gets rid of ku that we don't care about
tmp3 <- tmp2[tmp2[,3] != "分区",]  #ku splitting is irrelevant
tmp4 <- tmp3[tmp3[,3] != "政令市（区制）",]  #only happens post-merger (handle via a different system?)
tmp5 <- tmp4[tmp4[,3] != "政令指定都市",]  #same
tmp6 <- tmp5[tmp5[,3] != "郡の新設",]  #we don't track gun
tmp7 <- tmp6[tmp6[,3] != "郡の統合",]  #same
henkou <- tmp7[tmp7[,3] != "町名変更",]  #only change is in pronunciation


jis <- henkou[,"ＪＩＳコード"]
jis <- gsub("９", "9", jis)
jis <- gsub("８", "8", jis)
jis <- gsub("７", "7", jis)
jis <- gsub("６", "6", jis)
jis <- gsub("５", "5", jis)
jis <- gsub("４", "4", jis)
jis <- gsub("３", "3", jis)
jis <- gsub("２", "2", jis)
jis <- gsub("１", "1", jis)
jis <- gsub("０", "0", jis)
henkou$jis <- jis

tmp <- henkou[,"実施年月日"]
henkou$date <- as.Date(tmp, "%y.%m.%d")

change.types <- c("市制", "町制", "支庁制度改革(JISコードの変更)", "村名変更", "町名変更",
                  "町名変更・市制", "郡の区域変更", "county.merger")
shichousei <- henkou[henkou[,3] %in% change.types,]
m.string <- as.character(shichousei[,2])
pref.end <- rep(3, length(m.string))
pref.end[substring(m.string, first=1, last=4) %in% c("和歌山県", "神奈川県", "鹿児島県")] <- 4
pref <- substring(m.string, first=1, last=pref.end)
notpref <- substring(m.string, first=pref.end+1)
county.end <- regexpr("郡", notpref)
county <- substring(notpref, first=1, last=county.end)
muni.end <- regexpr("[（(]", notpref) - 1
muni.end[muni.end < 0] <- nchar(notpref[muni.end < 0])
muni <- substring(notpref, first=county.end+1, last=muni.end)

changes <- data.frame(date=shichousei$date, pref, county, muni, row.names=m.string, stringsAsFactors=FALSE)
changes$jis.old <- jis.by.muni[paste(pref, county, muni, sep="、")]
changes$type <- shichousei[,3]
changes$county.new <- changes$county
changes[shichousei[,3] %in% c("市制", "町名変更・市制"), "county.new"] <- ""
tmp <- shichousei[,4]
county.end <- regexpr("郡", tmp)
use.county <- shichousei[,3] %in% c("郡の区域変更", "county.merger")
changes[use.county, "county.new"] <- substring(tmp[use.county], first=1, last=county.end[use.county])
muni.new.str <- tmp
muni.new.str[use.county] <- substring(tmp[use.county], first=county.end[use.county]+1)
muni.new.end <- regexpr("[（(]", muni.new.str)
changes$muni.new <- substring(muni.new.str, first=1, last=muni.new.end-1)
changes$jis.new <- shichousei$jis
jis.by.muni.chgd <- as.character(changes$jis.old)
names(jis.by.muni.chgd) <- paste(changes$pref, changes$county.new, changes$muni.new, sep="、")


###begin mergers
tmp <- henkou[!(henkou[,3] %in% change.types),]
gappei <- tmp[order(tmp$date),]
m.string <- gappei[,"旧市町村名"]
pref.end <- attr(regexpr("..+?[都道府県]", m.string), "match.length")
pref <- substring(m.string, first=1, last=pref.end)
notpref <- substring(m.string, first=pref.end+1)
munis.str <- strsplit(notpref, "、")

get.jis <- function(lookup.str) {
  ret <- jis.by.muni[lookup.str]
  tmp <- jis.by.muni.chgd[lookup.str]
  names(ret)[is.na(ret)] <- lookup.str[is.na(ret)]
  ret[is.na(ret)] <- tmp[is.na(ret)]
  return(ret)
}

do.get.jis <- function(munis, pref) {
  is.city <- grepl("市$", munis)
  county.end <- regexpr("(?<=.)郡", munis, perl=TRUE)
  county <- substring(munis, first=1, last=county.end)
  county[is.city] <- ""
  notcounty <- substring(munis, first=county.end+1)
  notcounty[is.city] <- munis[is.city]
  muni.l <- strsplit(notcounty, "・")
  lookup.str.l <- mapply(paste, rep(pref, length(county)), county, muni.l, MoreArgs=list(sep="、"))
  return(get.jis(unlist(lookup.str.l)))
}

m.l <- mapply(do.get.jis, munis.str, pref)

## ok, now look at the "target" column, and update "main" prefecture and county as appropriate
county <- shp@data[sapply(m.l, function(x) x[1]), "county"]
do.col <- function(dat) { # <<- side effects!
  pref.end <- attr(regexpr("..県", dat), "match.length")   #looks like there's only one merger across prefectural boundaries (target col)
  new.pref <- substring(dat, first=1, last=pref.end)
  pref[pref.end > 0] <<- new.pref[pref.end > 0]
  notpref <- substring(dat, first=pref.end+1)
  county.end <-  attr(regexpr(".+?郡", notpref), "match.length")   #seem to be three of these in target column
  new.county <- substring(notpref, first=1, last=county.end)
  is.city <- grepl("市$", notpref) #need to blank out counties for the targets that are cities
  update.county <- nchar(new.county) > 0 | is.city
  county[update.county] <<- new.county[update.county]
  notcounty <- substring(notpref, first=county.end+1)
  return(notcounty)
}

#do the "target" column (with auto updating of pref and county)
tmp <- gappei[,"合併等の内容"]
target.end <- regexpr("へ", tmp)
notcounty <- do.col(substring(tmp, first=1, last=target.end-1))
target <- paste(pref, county, notcounty, sep="、")
target[target.end==-1] <- NA

#do the "outcome" column (")
tmp <- gappei[,"合併.市.町制施行後の状況"]
outcome.end <- regexpr("[（(]", tmp)
col.input <- substring(tmp, first=1, last=outcome.end-1)
col.input[outcome.end==-1] <- tmp[outcome.end==-1]
notcounty <- do.col(col.input)
gappei$merger.name <- paste(pref, county, notcounty, sep="、")

gappei$merger <- vector("list", dim(gappei)[1])
#doesn't seem to be any reasonable way to do this other than sequentially...
for (i in 1:length(m.l)) {
  if(any(names(m.l[[i]]) %in% gappei[0:(i-1), "merger.name"])) stop("remerger")
  if(gappei[i, "merger.name"] %in% gappei[0:(i-1), "merger.name"] & target[i] != gappei[i, "merger.name"]) stop("duplicate merger?")
  if(!is.na(target[i])) {
    if(!is.na(rev.idx <- match(target[i], gappei[(i-1):0, "merger.name"]))) { #"target remerger"
      old.merger <- gappei$merger[[i-rev.idx]]
      gappei$merger[[i]] <- old.merger
    } else gappei$merger[[i]] <- get.jis(target[i])
  }
  gappei$merger[[i]] <- c(gappei$merger[[i]], m.l[[i]])
}
gappei[gappei[,"合併等の内容"]=="岐阜県中津川市へ編入", "date"] <- "2005-02-12"
row.names(gappei) <- paste(gappei$jis, gappei$date, sep=".")
save(gappei, changes, file=paste(proj.home, "gappei.RData", sep=""))
