## ./greedy.R
##
## Produces results regarding centralized mergers, based on a "greedy" approximation algorithm 
## Runs on one prefecture at a time.  For all of Japan (47 prefectures), submit as

#bsub -n 8,20 -R "span[hosts=1]" -W 24:00 -J greedy[1-47] Rscript ./greedy.R actual

fname <- paste(proj.home, "anal_base.R", sep="")
source(fname)

component.obj <- function(gr, rt="all") {
    stopifnot(rt %in% c("all", "T"))
    munis <- V(gr)$name
    if(!VARONLY) {
        comp.dat <- calc.dat(munis)
        v <- v.m.new(d=comp.dat, b=b, time=0, ninv=ninv, T0subsidy=FALSE) * pop.muni[munis]
        T <- v.m.new(d=comp.dat, b=b, time=0, ret="T", T0subsidy=FALSE)
        stopifnot(T == T[1])
    }
    if(VAR) {
        var.term <- sqrt(sum(pop.muni[munis])) * sum(var.muni[munis] * pop.muni[munis]) / sum(pop.muni[munis])
        if(!VARONLY) {
            u <- (v.m.new(d=comp.dat, b=b, time=0, T0subsidy=FALSE) - var.term) * pop.muni[munis]
            ret <- sum(u * weight.pc[munis]) - cpf * T[1]
        } else ret <- sum(-var.term * weight.pc[munis] * pop.muni[munis])
    } else ret <- sum(v * weight.pc[munis]) - cpf * T[1]
    if(rt=="all") return(ret)
    if(rt=="T") return(T[1])
}

edge.benefit <- function(enum) {
  g.l.base <- decompose.graph(graph.base)
  graph.try <- graph.base + edges(V(p.graph)[ adj(E(p.graph)[enum]) ])
  g.l.try <- decompose.graph(graph.try)
  removed <- !(g.l.base %in% g.l.try)
  if (sum(removed) == 0) return(0)
  obj.removed <- sum(sapply(g.l.base[removed], component.obj))
  added <- !(g.l.try %in% g.l.base)
  obj.added <- sum(sapply(g.l.try[added], component.obj))
  return(obj.added - obj.removed)
}

library("ucminf")
library("parallel")
graph <- p.graph - E(p.graph)
graph.base <- graph
repeat {
    benefit <- unlist(mclapply(as.list(1:ecount(p.graph)), edge.benefit))
    add.edge <- which.max(benefit)
    if(benefit[add.edge] <= 0) break
    graph.base <- graph.base + edges(V(p.graph)[ adj(E(p.graph)[add.edge]) ])
    stopifnot(!any(duplicated(V(graph.base)$name)))
}
part.l <- lapply(decompose.graph(graph.base), V)

g.l.singleton <- decompose.graph(graph)
obj.singleton <- sum(sapply(g.l.singleton, component.obj))
g.l.centralized <- decompose.graph(graph.base)
obj.centralized <- sum(sapply(g.l.centralized, component.obj))

if(!VARONLY) {
    T.singleton <- sum(sapply(g.l.singleton, component.obj, rt="T"))
    T.centralized <- sum(sapply(g.l.centralized, component.obj, rt="T"))
}

fname <- paste(proj.tmp, "greedy_", type, "_", p, "_", as.integer(Sys.time()), ".RData", sep="")
if(!VARONLY) {
    if(VAR) {
        save(part.l, obj.singleton, obj.centralized, T.singleton, T.centralized, var.muni, mean.muni, weight.pc, file=fname)
    }
    save(part.l, obj.singleton, obj.centralized, T.singleton, T.centralized, weight.pc, file=fname)
} else save(part.l, obj.singleton, obj.centralized, weight.pc, file=fname)


print(part.l)

