## ./individ_im.R
## Create the individual data files, on a per player basis.  Submit via

#qsub -N im_1 -v PREF=1,LONG=1 -t 12-16 -q fas_normal -l walltime=168:00:00 individ_im.qscript
#qsub -N im_1 -v PREF=1,LONG=0 -t 1-16 -q fas_normal -l walltime=24:00:00 individ_im.qscript

proj.home <- "/home2/egw5/japan_restud/"
proj.tmp <- "/home2/egw5/scratch/japan_restud/"

options(echo=TRUE)
tmp <- commandArgs(trailingOnly=TRUE)
tmp
p <- as.numeric(tmp[1])
p
size <- as.numeric(tmp[2])
size
long <- as.numeric(tmp[3])
long

library("igraph0")  #legacy ZERO-BASED INDEXING
library("bigmemory")
library("biganalytics")
options(bigmemory.typecast.warning=FALSE)

load(paste(proj.home, "mesh.RData", sep=""))  #should include get.med()
load(paste(proj.home, "pgraph.RData", sep=""))

p.graph <- p.graph.l[[p]]
all.munis <- V(p.graph)$name
nmuni <- length(all.munis)

if(long && size==1) {
  coals <- as.big.matrix(matrix((1:nmuni) - 1, ncol=1), type="short", shared=FALSE)
} else if (long && size==2) {
  coals <- as.big.matrix(t(sapply(as.list(0:(ecount(p.graph)-1)), function(x) get.edge(p.graph, x))), type="short", shared=FALSE)
} else {
  if(long) {
    fname <- paste(proj.tmp, p, "/", size, "_long.csv", sep="")
  } else fname <- paste(proj.tmp, p, "/", size, "_short.csv", sep="")
  f.bytes <- file.info(fname)[,"size"]
  if(is.na(f.bytes)) stop("missing file")
  coals <- if(f.bytes == 0) matrix(NA, nrow=0, ncol=size) else read.big.matrix(fname, type="short", shared=FALSE)
}
length(coals) * 2


rm(all.munis, nmuni, fname, f.bytes)

library("sp")
igraphnp1.to.jis <- V(p.graph)$name
cells.by.igraphnp1 <- cells[igraphnp1.to.jis]
pop.by.igraphnp1 <- sapply(cells.by.igraphnp1, function(x) sum(x[,"adj.pop"]))


f <- function(i) {
  coal <- coals[i,]
  coal.avg.dist <- rep(NA, size)
  med <- get.med(do.call("rbind", cells.by.igraphnp1[coal+1]))  #should come from mesh.RData
  for(j in 1:size) {
    tmp <- cells.by.igraphnp1[[coal[j] + 1]][,c("lon", "lat", "adj.pop")]
    tmp2 <- tmp[tmp[,"adj.pop"] > 0,,drop=FALSE]
    stopifnot(nrow(tmp2) > 0)
    if(nrow(tmp2) > 1) {
      coal.avg.dist[j] <- weighted.mean(spDistsN1(tmp2[,c("lon", "lat")], med, longlat=TRUE), tmp2[,"adj.pop"])
    } else coal.avg.dist[j] <- 0
  }
  return(coal.avg.dist)
}

library("orloca")
library("parallel")
if(nrow(coals) > 0) system.time(tmp <- mclapply(1:nrow(coals), f, mc.cores=7)) else tmp <- list(coals)
rm(cells, cells.by.igraphnp1, coals, f, get.med, igraphnp1.to.jis, mergers.l, mesh.r, p.graph, p.graph.l, pop.by.igraphnp1)
ls()
gc()
object.size(tmp)

avg.dist.m <- do.call("rbind", tmp)
rm(tmp)
gc()


if(long) {
  fname <- paste(proj.tmp, p, "/i_long_", size, "_dist.RData", sep="")
} else fname <- paste(proj.tmp, p, "/i_short_", size, "_dist.RData", sep="")
tmp.fname <- paste(proj.tmp, p, "/tmp_", size, "_dist.RData", sep="")
save(avg.dist.m, file=tmp.fname)
rm(avg.dist.m)
if(size > 10) Sys.sleep(60) #something funny was happening with very large files...
stopifnot(file.rename(tmp.fname, fname))

