#sqCreateScript -w 168:00:00 10_mesh.task1 | qsub

proj.home <- "/home2/egw5/japan_restud/"
proj.tmp <- "/home2/egw5/scratch/japan_restud/"
options(echo=TRUE)

# create mesh, assign mesh cells to municipalities
fname <- paste(proj.home, "mesh_95.txt", sep="")
tmp <- read.csv(file(fname, encoding="SHIFT-JIS"), row.names=1)
mesh <- tmp[nchar(row.names(tmp)) == 8,]
lat.1 <- as.numeric(substr(rownames(mesh), start=1, stop=2))
lon.1 <- as.numeric(substr(rownames(mesh), start=3, stop=4))
lat.2 <- as.numeric(substr(rownames(mesh), start=5, stop=5))
lon.2 <- as.numeric(substr(rownames(mesh), start=6, stop=6))
lat.3 <- as.numeric(substr(rownames(mesh), start=7, stop=7))
lon.3 <- as.numeric(substr(rownames(mesh), start=8, stop=8))

#convert to new coordinates, using 3600 (i.e. 24 deg.) as the starting latitude, and 2200 (i.e. 122 deg.) as the starting longitude
Y.DEG.HEIGHT <- (1/1.5) / 80
X.DEG.WIDTH <- 1/80
#new conversion, to get the correct orientation
mesh$y <- (68 - lat.1) * 80 + (7 - lat.2) * 10 + (9 - lat.3) + 1   #puts 6879 on the very first row, 3600 at the bottom
mesh$x <- (lon.1 - 22) * 80 + lon.2 * 10 + lon.3 + 1    #puts 2200 on the first column, 5379 at the far right

library("Matrix")
mesh.sm <- sparseMatrix(i=mesh$y, j=mesh$x, dims=c(33 * 80, 32 * 80), x=mesh[,2])

library("raster")
mesh.r <- raster(as.matrix(mesh.sm), xmn=122, xmx=122 + ncol(mesh.sm) * X.DEG.WIDTH,
                 ymn=69/1.5 - nrow(mesh.sm) * Y.DEG.HEIGHT, ymx=69/1.5)
projection(mesh.r) <- "+proj=lonlat +ellps=bessel +towgs84=-147.54,507.26,680.47"
writeRaster(mesh.r, paste(proj.home, "mesh.grd", sep=""), overwrite=TRUE)

load(paste(proj.home, "shp.RData", sep=""))
values.out <- extract(mesh.r, shp, weights=TRUE, cellnumbers=TRUE)
#values.out <- polygonValues(shp, mesh.r, weights=TRUE, cellnumbers=TRUE) #takes over 9hrs, apparently
#save(values.out, file=paste(proj.home, "mesh_values.RData", sep=""))
#load("/home2/egw5/japan_new/mesh_values.RData")

#adjust for the fact that coastline cells won't be completely covered
coverage.m <- do.call("rbind", values.out)
coverage <- aggregate(coverage.m[,"weight"], by=list(cell=coverage.m[,"cell"]), FUN=sum)

f <- function(x) {
  if(is.null(x)) return(NULL)
  total.weight <- coverage[match(x[,"cell"], coverage[,"cell"]), 2]
  adj.pop <- x[,"value"] * x[,"weight"] / total.weight
  lon <- xFromCell(mesh.r, x[,"cell"])
  lat <- yFromCell(mesh.r, x[,"cell"])
  ret <- cbind(pop=x[,"value"], weight=x[,"weight"], total.weight=total.weight, adj.pop=adj.pop, lon=lon, lat=lat)
  rownames(ret) <- x[,"cell"]
  return(ret)
}
#final adjusted numbers
cells <- lapply(values.out, f) #should take about 3min.
names(cells) <- row.names(shp@data)

get.med <- function(cells.muni) {
  tmp <- cells.muni[cells.muni[,"adj.pop"] > 0, c("lon", "lat", "adj.pop"), drop=FALSE]
  mean.y <- mean.lat <- weighted.mean(tmp[,"lat"], tmp[,"adj.pop"])
  lon.scale <- cos((mean.lat/360)*2*pi)  # rescale longitude so a degree is the same length as latitude
  loca <- loca.p(x=tmp[,"lon"] * lon.scale, y=tmp[,"lat"], w=tmp[,"adj.pop"])
  mean.x <- weighted.mean(loca@x, loca@w)
  med.xy <- zsummin(loca, x=mean.x, y=mean.y, algorithm="ucminf") #seems to be fastest algorithm
  return(med.xy / c(lon.scale, 1))
}

save(get.med, mesh.r, cells, file=paste(proj.home, "mesh.RData", sep=""))

