# Rscript --verbose /home/fas/weese/egw5/japan_qe_final/other_merger.R
# Provides Figures 17 and 18

options(echo=TRUE)

CORES <- as.numeric(Sys.getenv("LSB_DJOB_NUMPROC")) - 1
if(is.na(CORES) || CORES < 2) CORES <- 1
options(mc.cores=CORES)

proj.home <- "/home/fas/weese/egw5/japan_qe_final/"
proj.tmp <- "/home/fas/weese/egw5/scratch/japan_qe/"

fname <- paste(proj.home, "fin_h13.RData", sep="")
load(fname)
rownames(df.h13) <- as.character(as.numeric(rownames(df.h13)))
rownames(num.units) <- as.character(as.numeric(rownames(num.units)))

fname <- paste(proj.home, "mesh.RData", sep="")
load(fname)

fname <- paste(proj.home, "hosei.R", sep="")
source(fname)

fname <- paste(proj.home, "toku_inv.R", sep="")
source(fname)

fname <- paste(proj.home, "calc_dat.R", sep="")
source(fname)

par.calibrated <- c(B0=NA, B1=NA, B2=NA, B3=1, B4=1)
par.calibrated["tau.bar"] <- 0.084 * 4/3

par.calibrated["r"] <- 0.02
par.calibrated["disc.santei.h20"] <- par.calibrated["r"] * sum(c(1/(1+par.calibrated["r"])^(1:10), c(.9, .7, .5, .3, .1)/(1+par.calibrated["r"])^(11:15)))
par.calibrated["disc.tokurei"] <- par.calibrated["r"] * sum(1/(1+par.calibrated["r"])^(1:10)) * 0.95 * 0.70 * (sum(1/(1+par.calibrated["r"])^(1:20))/20) / 10

library("sp")
library("ucminf")
library("parallel")
calc.merger <- function(ml, mal, mbl) {
##    d <- do.call("rbind", mclapply(ml, calc.dat, mc.cores=CORES))
##    da.single <- do.call("rbind", mclapply(mal, calc.dat, mc.cores=CORES))
##    db.single <- do.call("rbind", mclapply(mbl, calc.dat, mc.cores=CORES))
    d <- do.call("rbind", lapply(ml, function(x) calc.dat(x)[1,,drop=FALSE]))
    da.single <- do.call("rbind", lapply(mal, calc.dat))
    db.single <- do.call("rbind", lapply(mbl, calc.dat))
    subsidy <- d[,"sfn.subsidy"] + pmin(0, d[,"sfn.idio"])
    subsidy.a.single <- da.single[,"sfn.subsidy"] + pmin(0, da.single[,"sfn.idio"])
    subsidy.b.single <- db.single[,"sfn.subsidy"] + pmin(0, db.single[,"sfn.idio"])
    zeta <- pmax(0, d[,"sfn.idio"])
    zeta.a.single <- pmax(0, da.single[,"sfn.idio"])
    zeta.b.single <- pmax(0, db.single[,"sfn.idio"])

    T.pre <- pmax(0, d[,"sfn.struct.h13"] + zeta - d[,"sfr"] + subsidy) - subsidy
    T.pre.a.single <- pmax(0, da.single[,"sfn.struct.h13"] + zeta.a.single - da.single[,"sfr"] + subsidy.a.single) - subsidy.a.single
    T.pre.b.single <- pmax(0, db.single[,"sfn.struct.h13"] + zeta.b.single - db.single[,"sfr"] + subsidy.b.single) - subsidy.b.single
    T.post <- pmax(0, d[,"sfn.struct.h20"] + zeta - d[,"sfr"] + par.calibrated["disc.tokurei"] * d[,"tokureisai"] + subsidy) - subsidy + par.calibrated["disc.santei.h20"] * d[,"santei.h20"]
    T.post.a.single <- pmax(0, da.single[,"sfn.struct.h20"] + zeta.a.single - da.single[,"sfr"] + subsidy.a.single) - subsidy.a.single
    T.post.b.single <- pmax(0, db.single[,"sfn.struct.h20"] + zeta.b.single - db.single[,"sfr"] + subsidy.b.single) - subsidy.b.single
#    return(c(pre=T.pre, post=T.post, pop=d["pop"]/2, sfn.struct=d["sfn.struct.h13"]+zeta))
    return(cbind(pre=T.pre, post=T.post, pop=d[,"pop"]/2, sfn.struct=d[,"sfn.struct.h13"]+zeta, pop.a=da.single[,"pop"], pop.b=db.single[,"pop"], toku=par.calibrated["disc.tokurei"]*d[,"tokureisai"], santei=par.calibrated["disc.santei.h20"] * d[,"santei.h20"], pre.a.single=T.pre.a.single, post.a.single=T.post.a.single, pre.b.single=T.pre.b.single, post.b.single=T.post.b.single))
}

SIZE <- 100
SAMPLES <- 25

z <- matrix(NA, ncol=SIZE, nrow=SIZE)

pmin <- min(df.h13[,"pop"])
pmax <- max(df.h13[,"pop"])

s.thresh <- exp(seq(from=log(200), to=log(3500000), length.out=SIZE+1))

g <- function(x,y)
    row.names(df.h13)[c(x,y)]

f <- function(i,j) {
    in.i <- df.h13[,"pop"] > s.thresh[i] & df.h13[,"pop"] < s.thresh[i+1]
    in.j <- df.h13[,"pop"] > s.thresh[j] & df.h13[,"pop"] < s.thresh[j+1]
    if(sum(in.i) == 0 || sum(in.j) == 0) return(NA)

    if(sum(in.i) > 1) {
        use.i <- sample(which(in.i), SAMPLES, replace=TRUE)
    } else use.i <- rep(which(in.i), length.out=SAMPLES)
    if(sum(in.j) > 1) {
        use.j <- sample(which(in.j), SAMPLES, replace=TRUE)
    } else use.j <- rep(which(in.j), length.out=SAMPLES)

#    munis.l.tmp <- lapply(1:SAMPLES, function(x) c(use.i[x], use.j[x]))
    munis.l.tmp <- lapply(1:SAMPLES, function(x) c(row.names(df.h13)[use.i[x]], row.names(df.h13)[use.j[x]]))
    munis.l <- munis.l.tmp[!duplicated(munis.l.tmp)]
    munis.a.l <- lapply(munis.l, function(x) x[1])
    munis.b.l <- lapply(munis.l, function(x) x[2])
    out <- calc.merger(munis.l, munis.a.l, munis.b.l)
    stopifnot(nrow(out)==length(munis.l))
    incentive <- (out[,9] - out[,10])+(out[,11]-out[,12])+(out[,2] - out[,1])
    delta.T <- out[,9] + out[,11] - out[,1]
    ifrac <- pmax(-1, pmin(2, incentive/delta.T))
    ifrac[incentive==0 & delta.T==0] <- 1
    return(mean(ifrac))
}

for(i in 1:SIZE)
    z[i,i:SIZE] <- unlist(mclapply(i:SIZE, f, i, mc.cores=CORES))

z[lower.tri(z)] <- t(z)[lower.tri(z)]

s.omit <- apply(apply(z, 1, is.na), 1, all)
##tmp <- expand.grid(s.thresh[1:SIZE], s.thresh[1:SIZE])
tmp <- expand.grid(s.thresh[which(!s.omit)], s.thresh[which(!s.omit)])

library("lattice")

z.censored <- apply(z, 1:2, function(x) pmax(0, pmin(1, x)))

fname <- paste(proj.home, "pairs_incentive.pdf", sep="")
pdf(file=fname)
##levelplot(z.censored[1:SIZE,1:SIZE]~tmp[,1]*tmp[,2], scales=list(x=list(log=TRUE), y=list(log=TRUE)), xlab="Population", ylab="Population", panel = function(...) {
levelplot(z.censored[!s.omit,!s.omit]~tmp[,1]*tmp[,2], scales=list(x=list(log=TRUE), y=list(log=TRUE)), xlab="Population", ylab="Population", panel = function(...) {
##    panel.fill(col = "grey")
    panel.levelplot(...)
})
dev.off()

fname <- paste(proj.home, "pairs_incentive_bw.pdf", sep="")
pdf(file=fname)
levelplot(z.censored[!s.omit,!s.omit]~tmp[,1]*tmp[,2], scales=list(x=list(log=TRUE), y=list(log=TRUE)), xlab="Population", ylab="Population", col.regions = gray(0:255/255))
dev.off()



######
fname <- paste(proj.home, "pgraph.RData", sep="")
load(fname)
rm(fname)

do.one <- function(munis) {
    d <- calc.dat(munis)
    d.single <- do.call("rbind", lapply(munis, calc.dat))
    subsidy <- d[1,"sfn.subsidy"] + pmin(0, d[1,"sfn.idio"])
    subsidy.single <- d.single[,"sfn.subsidy"] + pmin(0, d.single[,"sfn.idio"])
    zeta <- pmax(0, d[1,"sfn.idio"])
    zeta.single <- pmax(0, d.single[,"sfn.idio"])

    T.pre <- pmax(0, d[1,"sfn.struct.h13"] + zeta - d[1,"sfr"] + subsidy) - subsidy
    T.pre.single <- pmax(0, d.single[,"sfn.struct.h13"] + zeta.single - d.single[,"sfr"] + subsidy.single) - subsidy.single
    T.post <- pmax(0, d[1,"sfn.struct.h20"] + zeta - d[1,"sfr"] + par.calibrated["disc.tokurei"] * d[1,"tokureisai"] + subsidy) - subsidy + par.calibrated["disc.santei.h20"] * d[1,"santei.h20"]
    T.post.single <- pmax(0, d.single[,"sfn.struct.h20"] + zeta.single - d.single[,"sfr"] + subsidy.single) - subsidy.single

    incentive <- sum(T.pre.single - T.post.single) + (T.post - T.pre)
    delta.T <- sum(T.pre.single) - T.pre
    if(incentive == 0 & delta.T == 0) return(1)
    return(incentive/delta.T)
}

actual.incentive <- sapply(mergers.l, do.one)

fname <- paste(proj.home, "incentive_actual.pdf", sep="")
pdf(file=fname)
densityplot(actual.incentive, plot.points=FALSE, xlab="Incentive")
dev.off()

fname <- paste(proj.home, "incentive_actual_bw.pdf", sep="")
pdf(file=fname)
densityplot(actual.incentive, plot.points=FALSE, xlab="Incentive", col="black")
dev.off()
