# ./pairs.R
## Run simulations for the analysis in Section 5  
## Submit as an array job, with the prefectures as the array ids.  For all of Japan, run as

#bsub -n 5,10 -R "span[hosts=1]" -W 24:00 -J pairs[1-47] Rscript ./pairs.R actual

TAX.FLOOR <- TRUE # use the model from Column I of Table 2

NODIST <- FALSE  # (replace distance with average distance)
NOX <- FALSE # (replace characteristics with average characteristics)

proj.home <- "/home/fas/weese/egw5/japan_qe/"
proj.tmp <- "/home/fas/weese/egw5/scratch/japan_qe/"
prefs <- 1:47

options(echo=TRUE)
tmp.args <- commandArgs(trailingOnly=TRUE)
tmp.args

type <- tmp.args[1]
stopifnot(type %in% c("actual", "b34", "forever", "equal", "ninv", "nodist", "iden", "nocorr", "fullcorr"))
ninv <- type %in% c("equal", "ninv", "nodist", "iden", "nocorr", "fullcorr")
fixedd <- type != "nodist"
pnum <- as.numeric(tmp.args[2])
if(is.na(pnum)) {
    pnum <- as.numeric(Sys.getenv("LSB_JOBINDEX"))
}
pnum
cpf <- 1

CORES <- as.numeric(Sys.getenv("LSB_DJOB_NUMPROC")) - 1
if(is.na(CORES) || CORES < 2) CORES <- 1
options(mc.cores=CORES)

par.calibrated <- c(B0=NA, B1=NA, B2=NA, B3=1, B4=1)
par.calibrated["tau.bar"] <- 0.084 * 4/3
par.calibrated["r"] <- 0.02

par.calibrated["disc.santei.h13"] <- 0
if(type == "forever") {
    par.calibrated["disc.santei.h20"] <- 1
    par.calibrated["disc.tokurei"] <- (1/10) * 0.95 * 0.70 * sum(1/(1+par.calibrated["r"])^(1:20))/20
} else {
    par.calibrated["disc.santei.h20"] <- par.calibrated["r"] * sum(c(1/(1+par.calibrated["r"])^(1:10), c(.9, .7, .5, .3, .1)/(1+par.calibrated["r"])^(11:15)))
    par.calibrated["disc.tokurei"] <- (1/10) * par.calibrated["r"] * sum(1/(1+par.calibrated["r"])^(1:10)) * 0.95 * 0.70 * (sum(1/(1+par.calibrated["r"])^(1:20))/20)
}

if(type == "b34") {
    par.estimated <- c(B2=0.2164504, B0=219.5554103, B1=5.0432098, B4=1, B3=1)
} else {
    par.estimated <- c(B2=0.2164504, B0=219.5554103, B1=5.0432098, B4=0.4882909, B3=1.0446544)
}

b <- par.calibrated
b[names(par.estimated)] <- par.estimated

fname <- paste(proj.home, "h13.RData", sep="")
load(fname)
rownames(d.h13) <- as.character(as.numeric(rownames(d.h13)))

library("igraph") #ONE-BASED indexing
fname <- paste(proj.home, "pgraph.RData", sep="")
load(fname)



fname <- paste(proj.home, "fin_h13.RData", sep="")
load(fname)
rownames(df.h13) <- as.character(as.numeric(rownames(df.h13)))
rownames(num.units) <- as.character(as.numeric(rownames(num.units)))

fname <- paste(proj.home, "mesh.RData", sep="")
load(fname)
rm(fname)

fname <- paste(proj.home, "hosei.R", sep="")
source(fname)

fname <- paste(proj.home, "v.R", sep="")
source(fname)

fname <- paste(proj.home, "toku_inv.R", sep="")
source(fname)

library("ucminf")
library("orloca")
library("sp")

if(NOX) {
    df.avg <- colSums(df.h13)/sum(df.h13[,"pop"])
    df.avg["density"] <- NA
    num.units.avg <- colSums(num.units)/sum(num.units[,1])
    fin.avg <- colSums(fin.h13)/sum(df.h13[,"pop"])
}


### modified substantially from other versions...
calc.dat <- function(muni.jis) {
  if(NOX) {
      coal.h13 <- df.avg * sum(df.h13[muni.jis,"pop"])
      coal.h13["density"] <- coal.h13["pop"] / coal.h13["area"] * 100
      coal.num.units <- num.units.avg * sum(df.h13[muni.jis,"pop"])
  } else {
      coal.h13 <- colSums(df.h13[muni.jis,])
      coal.num.units <- colSums(num.units[muni.jis,])
  }
  coal.coef.h13 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H13"]])))
  coal.coef.h20 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H20"]])))

  coal.sfn.one <- h13.unit.cost * coal.num.units
  coal.sfn.h13 <- h13.unit.cost * coal.num.units * coal.coef.h13
  coal.sfn.h20 <- h20.unit.cost * coal.num.units * coal.coef.h20

  coal.coef.h13.ninv <- coal.coef.h13
  coal.coef.h13.ninv[investment] <- 1
  coal.sfn.h13.ninv <- h13.unit.cost * coal.num.units * coal.coef.h13.ninv

  dat <- vector("numeric")
  dat["pop"] <- sum(df.h13[muni.jis, "pop"])
  dat["income"] <- NA
  if (NOX) {
      dat["sfr"] <- sum(df.h13[muni.jis, "pop"]*df.avg["sfr"]) / 100
  } else dat["sfr"] <- sum(fin.h13[muni.jis, "sfr"]) / 100  #change things into $1k equivalents
  dat["sfn.struct.one"] <- sum(coal.sfn.one) / 100 
  dat["sfn.struct.h13"] <- sum(coal.sfn.h13) / 100
  dat["sfn.struct.h13.ninv"] <- sum(coal.sfn.h13.ninv) / 100
  dat["sfn.struct.h20"] <- sum(coal.sfn.h20) / 100
  if (NOX) {
      dat["sfn.subsidy"] <- sum(df.h13[muni.jis,"pop"]) * (fin.avg["sfn"] - fin.avg["sfn.exsub"]) / 100
      dat["sfn.idio"] <- sum(df.h13[muni.jis,"pop"]) * (fin.avg["sfn.exsub"] - fin.avg["sfn.struct.h13"]) / 100
  } else {
      dat["sfn.subsidy"] <- sum(fin.h13[muni.jis,"sfn"] - fin.h13[muni.jis,"sfn.exsub"]) / 100
      dat["sfn.idio"] <- sum(fin.h13[muni.jis,"sfn.exsub"] - fin.h13[muni.jis,"sfn.struct.h13"]) / 100
  }

  coal.lat.h13 <- max(0, dat["sfn.struct.h13"] + dat["sfn.subsidy"] + dat["sfn.idio"] - dat["sfr"])
  coal.lat.h20 <- max(0, dat["sfn.struct.h20"] + dat["sfn.subsidy"] + dat["sfn.idio"] - dat["sfr"])

  dat["tokureisai"] <- tokureisai(df.h13[muni.jis, "pop"])
  # In principle this shouldn't be negative if there are always efficiencies of scale, but a few of the splines have increasing marginal cost in a few places...
  if(length(muni.jis) == 1) {
  dat["santei.h13"] <- 0
  dat["santei.h20"] <- 0
  } else {
      singletons <- do.call("rbind", lapply(as.list(muni.jis), calc.dat))
      singletons.lat.h13 <- pmax(0, singletons[,"sfn.struct.h13"] + singletons[,"sfn.subsidy"] + singletons[,"sfn.idio"] - singletons[,"sfr"])
      singletons.lat.h20 <- pmax(0, singletons[,"sfn.struct.h20"] + singletons[,"sfn.subsidy"] + singletons[,"sfn.idio"] - singletons[,"sfr"])
      dat["santei.h13"] <- max(0, sum(singletons.lat.h13) - coal.lat.h13)
      dat["santei.h20"] <- max(0, sum(singletons.lat.h20) - coal.lat.h20)
  }

  cells.tmp <- do.call(rbind, cells[muni.jis])
  med <- get.med(cells.tmp)  #should come from mesh.RData
  coal.avg.dist <- sapply(cells[muni.jis], function(x) weighted.mean(spDistsN1(x[,c("lon", "lat"),drop=FALSE], med, longlat=TRUE), x[,"adj.pop"]))

  tmp <- matrix(dat, nrow=length(muni.jis), ncol=length(dat), byrow=TRUE)
  ret <- cbind(tmp, coal.avg.dist)
  colnames(ret) <- c(names(dat), "dist")
  return(ret)
}


library("numDeriv")
munis <- row.names(df.h13)
d.l <- lapply(munis, calc.dat)
d <- do.call("rbind", d.l)

T.actual <- v.m.new(d=d, b=b, ret="Tpre")
dudT <- grad(v.m.new, T=T.actual, d=d, b=b, ninv=ninv) * d[,"pop"]
weight.pc <- 1/dudT

if(type == "equal" || type == "b34" || NOX) weight.pc <- weight.pc * 0 + weighted.mean(weight.pc, d[,"pop"])

library("parallel")
####
get.dist <- function(enum) {
    munis <- V(p.graph)[ adj(E(p.graph)[enum]) ]$name
    dat.single <- rbind(calc.dat(munis[1]), calc.dat(munis[2]))
    dat.merged <- calc.dat(munis)
    return(dat.single[,"dist"] - dat.merged[,"dist"])
}

if(NODIST) dist.avg.incr <- 4.187566
if(type %in% c("nodist", "iden")) {
    dist.l <- list()
    for (p in prefs) {
        p.graph <- p.graph.l[[p]]
        pop.muni <- d.h13[V(p.graph)$name, "pop"]
        tmp <- mclapply(1:ecount(p.graph), get.dist)
        dist.l[[p]] <- do.call("rbind", tmp)
    }
    dist.m <- do.call("rbind", dist.l)
    dist.mean <- colSums(dist.m)/nrow(dist.m)
    dist.cov <- cov(dist.m)
    dist.avg.incr <- -sum(dist.m)/prod(dim(dist.m))
}
####

SIG <- 1
library("truncnorm")
library("mvtnorm")
library("tmvtnorm")
do.one <- function(T.pc, munis, val="W", fixed.dist, sig=SIG, d.s, d.m) {
    if (type == "fullcorr") {
        B.eps <- diag(c(1,1))
        Sigma2 <- matrix(sig^2, nrow=2, ncol=2)
    } else if (type == "nocorr") {
        B.eps <- diag(c(1,1))
        Sigma2 <- diag(c(sig^2,sig^2))
    } else {
        B.eps <- rbind(pop.muni[munis], pop.muni[munis]) / sqrt(sum(pop.muni[munis])) - diag(sqrt(pop.muni[munis]))
        sigma.eps <- diag(sig^2/pop.muni[munis])
        Sigma2 <- B.eps %*% sigma.eps %*% t(B.eps)
    }

    if(!fixed.dist) {
        B <- diag(rep(1,4))
        B[1:2,1:2] <- B.eps
        B[1:2,3:4] <- diag(c(b["B2"],b["B2"]))
        sigma.tmp <- matrix(0,nrow=4,ncol=4)
        sigma.tmp[1:2,1:2] <- sigma.eps
        sigma.tmp[3:4,3:4] <- dist.cov
        Sigma4 <- B %*% sigma.tmp %*% t(B)
        mean4 <- as.vector(B %*% c(0, 0, dist.mean))
    }

    if(length(T.pc)==1) {
        T <- T.pc * sum(pop.muni[munis])
        names(T) <- "merged"
    } else if(length(T.pc)==2) {
        T <- T.pc * pop.muni[munis]
        T["merged"] <- sum(T)
    } else {
        T <- T.pc * c(sum(pop.muni[munis]), pop.muni[munis])
    }

    dat.single <- d.s
    dat.merged <- d.m

    if(val != "merged") {
        if(fixed.dist) {
            v.single <- v.m.new(d=dat.single, b=b, T=T[c("single1","single2")], ninv=ninv)
        } else {
            v.single <- v.m.new(d=dat.single, b=b, T=T[c("single1","single2")], ninv=ninv) + b["B2"] * dat.single[,"dist"]
        }
    }
    if (val=="single") return((sum(v.single * pop.muni[munis] * weight.pc[munis]) - cpf * (T["single1"] + T["single2"]))/sum(pop.muni[munis]))
    if (fixed.dist) {
        if(!NODIST) {
            v.merged <- v.m.new(d=dat.merged, b=b, T=T["merged"], ninv=ninv)
        } else {
            v.merged <- v.m.new(d=dat.merged, b=b, T=T["merged"], ninv=ninv) + b["B2"] * dat.merged[,"dist"] - b["B2"] * (dat.single[,"dist"] + dist.avg.incr)
        }
    } else {
        v.merged <- v.m.new(d=dat.merged, b=b, T=T["merged"], ninv=ninv) + b["B2"] * dat.merged[,"dist"]
    }
    if (val=="merged") return((sum(v.merged * pop.muni[munis] * weight.pc[munis]) - cpf * T["merged"])/sum(pop.muni[munis]))
    if (val=="Tminus") {
        tmp.single <- v.m.new(d=dat.single, b=b, T=T[c("single1","single2")], ret="Tminus", ninv=ninv)/pop.muni[munis]
        tmp.merged <- v.m.new(d=dat.merged, b=b, T=T["merged"], ret="Tminus", ninv=ninv)[1]/sum(pop.muni[munis])
        return(c(tmp.merged, tmp.single))
    }
    if(v.merged[1] == -Inf) {
        if(val == "W")
            ret <- (sum(v.single * pop.muni[munis] * weight.pc[munis]) - cpf * T["merged"]) / sum(pop.muni[munis])
        if(val == "p")
            ret <- 0
        return(ret)
    }

    if(fixed.dist) {
        lower <- v.single - v.merged
        p.merger <- pmvnorm(lower=lower, sigma=Sigma2)
    } else {
        lower <- c(v.single - v.merged, -Inf, -Inf)
        p.merger <- pmvnorm(lower=lower, mean=mean4, sigma=Sigma4)
    }
    if (type %in% c("nocorr", "fullcorr")) {
        A.single.eps <- diag(c(-1,-1))
        A.merger.eps <- diag(c(0,0))
    } else {
        A.merger.eps <- rbind(pop.muni[munis], pop.muni[munis]) / sqrt(sum(pop.muni[munis]))
        A.single.eps <- diag(pop.muni[munis] / sqrt(pop.muni[munis]))
    }
    if(fixed.dist) {
        if (p.merger == 0) {  # mean.merger is actually the mean of the difference... (ugh)
            mean.merger <- v.single - v.merged
            eps.merger <- A.merger.eps %*% solve(B.eps) %*% mean.merger
            eps.merger.notmerged <- c(0, 0)
            eps.single <- c(0, 0)
        } else if (p.merger == 1) {
            mean.nomerger <- v.merged - v.single
            eps.merger <- c(0, 0)
            eps.single <- A.single.eps %*% solve(B.eps) %*% mean.nomerger
        } else {
            if (type == "fullcorr") {
                mean.merger.tmp <- mtmvnorm(sigma=Sigma2[1,1,drop=FALSE], lower=max(lower), doComputeVariance=FALSE)$tmean
                mean.merger <- rep(mean.merger.tmp, 2)
            } else mean.merger <- mtmvnorm(sigma=Sigma2, lower=lower, doComputeVariance=FALSE)$tmean
            eps.merger <- A.merger.eps %*% solve(B.eps) %*% mean.merger
            eps.merger.notmerged <- A.merger.eps %*% solve(B.eps) %*% -mean.merger*p.merger/(1-p.merger)
            eps.single <- A.single.eps %*% solve(B.eps) %*% -mean.merger*p.merger/(1-p.merger)
        }
    } else {
        tmp <- matrix(0, ncol=4, nrow=2)
        tmp[1:2,1:2] <- A.merger.eps
        tmp[1,3] <- b["B2"]/2
        tmp[2,4] <- b["B2"]/2
        A.merger <- tmp
        tmp[1:2,1:2] <- A.single.eps
        tmp[1,3] <- -b["B2"]/2
        tmp[2,4] <- -b["B2"]/2
        A.single <- tmp
            
        mean.merger <- mtmvnorm(sigma=Sigma4, lower=lower, mean=mean4, doComputeVariance=FALSE)$tmean
        eps.merger <- A.merger %*% solve(B) %*% mean.merger
        eps.merger.notmerged <- A.merger %*% solve(B) %*% (mean4 - mean.merger*p.merger)/(1-p.merger)
        eps.single <- A.single %*% solve(B) %*% (mean4 - mean.merger*p.merger)/(1-p.merger)
    }
    u.merged <- (v.merged + eps.merger) * pop.muni[munis]
    if(p.merger < 1) u.merged.notmerged <- (v.merged + eps.merger.notmerged) * pop.muni[munis]
    u.single <- (v.single + eps.single) * pop.muni[munis]

    if (val == "social") {
        W.single <- sum(v.single * weight.pc[munis] * pop.muni[munis] - cpf * T[c("single1", "single2")])
        W.merged <- sum(v.merged * weight.pc[munis] * pop.muni[munis]) - cpf * T["merged"]
        W.sigma <- t(pop.muni[munis] * weight.pc[munis]) %*% Sigma2 %*% (pop.muni[munis] * weight.pc[munis])
        lower.social <- W.single - W.merged
        p.social <- pnorm(lower.social, sd=sqrt(W.sigma), lower.tail=FALSE)
        mean.merger.social <- etruncnorm(a=lower.social, sd=sqrt(W.sigma))
        if(mean.merger.social == Inf) mean.merger.social <- lower.social
        ## for now, assume that only the merger has the idiosyncratic shock
        ret <- (W.merged + mean.merger.social) * p.social + W.single * (1 - p.social)
        return(list(ret, p.social))
    }
    
    u.avg <- u.merged * p.merger + u.single * (1-p.merger)
    T.avg <- (T["merged"] * p.merger) + (T["single1"] + T["single2"]) * (1-p.merger)
    if(val == "W") {
        ret <- (sum(u.avg * weight.pc[munis]) - cpf * T.avg) / sum(pop.muni[munis])
    } else if (val == "p") {
        ret <- p.merger
    } else if (val == "force") {
        if(p.merger == 1) {
            ret <- -1
        } else ret <- u.single - u.merged.notmerged
    } else ret <- "bad val" 
    return(ret)
}



ct <- list(fnscale=-1, trace=0)
do.edge <- function(arg.tmp) {
    if(is.numeric(arg.tmp)) {
        enum <- arg.tmp
        munis <- V(p.graph)[ adj(E(p.graph)[enum]) ]$name
    } else munis <- rep(arg.tmp, length.out=2)
    d.s <- rbind(calc.dat(munis[1]), calc.dat(munis[2]))
    d.m <- calc.dat(munis)

    T.start <- c(single1=10, single2=10)
    opt.out1 <- optim(T.start, do.one, control=ct, munis=munis, fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    stopifnot(opt.out1$convergence==0)

    T.start <- c(merged=sum(opt.out1$par * pop.muni[munis])/sum(pop.muni[munis]), opt.out1$par)
    opt.out2.tmp <- optim(T.start, do.one, control=ct, munis=munis, fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    stopifnot(opt.out2.tmp$convergence==0 && opt.out2.tmp$value - opt.out1$value > -1e-5)

    opt.out.fba <- optim(opt.out1$par, do.one, control=ct, munis=munis, val="single", fixed.dist=TRUE, d.s=d.s, d.m=d.m)
    stopifnot(opt.out.fba$convergence==0)
    lb <- min(-1, -2*max(abs(opt.out.fba$par)))
    ub <- max(1, 2*max(abs(opt.out.fba$par)))
    opt.out.fbb <- optim(mean(opt.out.fba$par), do.one, method="Brent", lower=lb, upper=ub, control=ct, munis=munis, val="merged", fixed.dist=TRUE, d.s=d.s, d.m=d.m)
    stopifnot(opt.out.fbb$convergence==0 && opt.out.fbb$par > lb + 0.01 && opt.out.fbb$par < ub - 0.01)

    opt.out2 <- opt.out2.tmp
    T.start <- c(opt.out2$par[1], opt.out.fba$par[1], opt.out2$par[3])
    opt.out2.tmp2 <- optim(T.start, do.one, control=ct, munis=munis, fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    stopifnot(opt.out2.tmp2$convergence==0)
    if(opt.out2.tmp2$val >= opt.out2$val) opt.out2 <- opt.out2.tmp2

    T.start <- c(opt.out2$par[1:2], opt.out.fba$par[2])
    opt.out2.tmp3 <- optim(T.start, do.one, control=ct, munis=munis, fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    stopifnot(opt.out2.tmp3$convergence==0)
    if(opt.out2.tmp3$val >= opt.out2$val) opt.out2 <- opt.out2.tmp3

    T.start <- c(merged=opt.out.fbb$par, opt.out2$par[2:3])
    opt.out2.tmp4 <- optim(T.start, do.one, control=ct, munis=munis, fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    stopifnot(opt.out2.tmp4$convergence==0)
    if(opt.out2.tmp4$val >= opt.out2$val) opt.out2 <- opt.out2.tmp4
    
    act.single <- v.m.new(d=d.s, b=b, ret="Tpost")/pop.muni[munis]
    act.single.pre <- v.m.new(d=d.s, b=b, ret="Tpre")/pop.muni[munis]
    act.merged <- v.m.new(d=d.m, b=b, ret="Tpost")[1]/sum(pop.muni[munis])
    act.merged.pre <- v.m.new(d=d.m, b=b, ret="Tpre")[1]/sum(pop.muni[munis])
    
    par.opt <- rbind(sb=opt.out2$par, fb=c(opt.out.fbb$par, opt.out.fba$par), actualh13=c(act.merged.pre, act.single.pre), actual=c(act.merged, act.single))

    fb.single <-  do.one(T.pc=par.opt["fb",], munis, val="single", fixed.dist=TRUE, d.s=d.s, d.m=d.m)
    fb.merged <-  do.one(T.pc=par.opt["fb",], munis, val="merged", fixed.dist=TRUE, d.s=d.s, d.m=d.m)

    social.tmp <- do.one(T.pc=par.opt["fb",], munis, val="social", fixed.dist=TRUE, d.s=d.s, d.m=d.m)
    
    savings <- sum(pop.muni[munis] * par.opt["fb",2:3]) - sum(pop.muni[munis]) * par.opt["fb",1]
    stick <- rbind(sb=pop.muni[munis] * (par.opt["fb",2:3] - par.opt["sb",2:3]),
                   actual=pop.muni[munis] * (par.opt["actualh13",2:3] - par.opt["actual",2:3]))
    carrot <- rbind(sb=sum(pop.muni[munis]) * (par.opt["sb",1] - par.opt["fb",1]),
                    actual=sum(pop.muni[munis]) * (par.opt["actual",1] - par.opt["actualh13",1]))
    incentive <- rowSums(stick) + carrot
    incentive.frac <- (rowSums(stick) + carrot)/(sum(pop.muni[munis] * par.opt["fb",2:3]) - sum(pop.muni[munis]) * par.opt["fb",1])
    actual.savings <- (sum(pop.muni[munis] * par.opt["actualh13",2:3]) - sum(pop.muni[munis]) * par.opt["actualh13",1])
    ifa <- (rowSums(stick) + carrot)/actual.savings
    af <- max(0, min(1, ifa[2]*savings/incentive[1]))
    if (is.nan(af)) {
        prob.afrac <- NA
        val.afrac <- NA
    } else {
        prob.afrac <- do.one(T.pc=(1-af)*par.opt["fb",]+af*par.opt["sb",], munis, val="p", fixed.dist=TRUE, d.s=d.s, d.m=d.m)
        val.afrac <- do.one(T.pc=(1-af)*par.opt["fb",]+af*par.opt["sb",], munis, fixed.dist=TRUE, d.s=d.s, d.m=d.m)
    }
    prob.out <- rbind(sb=do.one(T.pc=par.opt["sb",], munis, val="p", fixed.dist=TRUE, d.s=d.s, d.m=d.m),
                      fb=do.one(T.pc=par.opt["fb",], munis, val="p", fixed.dist=TRUE, d.s=d.s, d.m=d.m),
                      afrac=prob.afrac,
                      social=social.tmp[[2]],
                      centralized=fb.merged > fb.single)
    val.out <- rbind(sb=do.one(T.pc=par.opt["sb",], munis, fixed.dist=TRUE, d.s=d.s, d.m=d.m),
                     fb=do.one(T.pc=par.opt["fb",], munis, fixed.dist=TRUE, d.s=d.s, d.m=d.m),
                     afrac=val.afrac,
                     social=social.tmp[[1]]/sum(pop.muni[munis]),
                     centralized=max(fb.merged, fb.single))

    force.u <- do.one(T.pc=par.opt["sb",], munis, val="force", fixed.dist=fixedd, d.s=d.s, d.m=d.m)
    force.budget <- sum(pop.muni[munis]) * par.opt["sb", 1] - sum(pop.muni[munis] * par.opt["sb", 2:3])
    force <- sum(force.u * weight.pc[munis]) + force.budget < 0
    return(list(pop=pop.muni[munis], par=par.opt, prob=prob.out, val=val.out, savings=savings, incentive=cbind(incentive, incentive.frac, ifa), stick=stick, carrot=carrot, force.u=force.u, force.budget=force.budget, force=force))
}


pop.muni <- d.h13[, "pop"]

if (type == "iden") {
    pref.str <- substr(names(pop.muni), start=1, stop=nchar(names(pop.muni)) - 3)
    use <- as.numeric(pref.str) == pnum
    opt.out <- mclapply(row.names(df.h13)[use], do.edge)
} else {
    p.graph <- p.graph.l[[pnum]]
    opt.out <- mclapply(1:ecount(p.graph), do.edge)
}


fname <- paste(proj.tmp, "opt_out5_", type, "_", pnum, ".RData", sep="")
save(opt.out, weight.pc, file=fname)


####
##prob.tmp <- lapply(opt.out.l, function(x) unlist(lapply(x, `[`, "prob")))
#sort(unlist(prob.tmp))




