/* #include "error.h"*/
#include <stdlib.h>
#include <time.h>

#include "MatrixAccessor.hpp"
extern "C" {
#include <R.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include <igraph.h>
  /* #include "mkl.h" */
}

#define MAX_DEPTH 14
#define NEGINF -1.0/0.0
/* #define BURN_IN_REPS 100 */ /* when we adjust the K_max scaling factor */
// #define TOTAL_REPS 100 /* includes above */
#define false 0

typedef struct {
  unsigned long long bits[4];
} muni_bitfield;


inline muni_bitfield mbf_and(const muni_bitfield x, const muni_bitfield y) {
  muni_bitfield ret;
  for(int i = 0; i < 4; i++)
    ret.bits[i] = x.bits[i] & y.bits[i];
  return(ret);
}

inline muni_bitfield mbf_or(const muni_bitfield x, const muni_bitfield y) {
  muni_bitfield ret;
  for(int i = 0; i < 4; i++)
    ret.bits[i] = x.bits[i] | y.bits[i];
  return(ret);
}

inline muni_bitfield mbf_not(const muni_bitfield x) {
  muni_bitfield ret;
  for(int i = 0; i < 4; i++)
    ret.bits[i] = ~x.bits[i];
  return(ret);
}

inline unsigned long long mbf_any(const muni_bitfield x) {
  return(x.bits[0] | x.bits[1] | x.bits[2] | x.bits[3]);
}

inline unsigned long long equal(const muni_bitfield x, const muni_bitfield y) {
  return(x.bits[0] == y.bits[0] && x.bits[1] == y.bits[1] && x.bits[2] == y.bits[2] && x.bits[3] == y.bits[3]);
}

double cp1[] = {1,1}; /* the leading one is to give something to divide off of... */
double cp2[] = {1,1,1};
double cp3[] = {1,1,1,1};
double cp4[] = {1,1,1,1,1};
double cp5[] = {1,1,1,1,1,1};
double cp6[] = {1,1,1,1,1,1,1};
double cp7[] = {1, .5, .5, .5, .5, .5, .5, .5};
double cp8[] = {1, .5, .25, .1, .1, .1, .1, .1, .1};
double cp9[] = {1, .5, .25, .1, .05, .025, .01, .005, .005, .005};
double cp10[] = {1,.25,.05,.01,.0025,.0025,.0025,.0025,.0025,.0025,.0025};
double cp11[] = {1,.25,.05,.01,.0025,.0025,.0025,.0025,.0025,.0025,.0025,.0025};
double cp12[] = {1,.25,.05,.01,.0025,.0025,.0025,.0025,.0025,.0025,.0025,.0025,.0025};
double cp13[] = {1,.25,.05,.01,.0025,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005};
double cp14[] = {1,.25,.05,.01,.0025,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005};
double cp15[] = {1,.25,.05,.01,.0025,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005,.0005};
double *cp_end[] = {&cp1[1], &cp2[2], &cp3[3], &cp4[4], &cp5[5], &cp6[6], &cp7[7], &cp8[8], &cp9[9],
		    &cp10[10], &cp11[11], &cp12[12], &cp13[13], &cp14[14], &cp15[15]};



igraph_vector_ptr_t *refinement(char *feasible_c, muni_bitfield *involved, muni_bitfield avail, muni_bitfield unused, int num_coal, int *num_m, int initial_coal);

void prep_involved(SEXP involved_sexp, muni_bitfield *involved, int num_coal, int num_muni) {
  unsigned long long one = 1;
  muni_bitfield mbf_zero = {{0, 0, 0, 0}};
  for (int j = 0; j < num_coal; j++) {
    involved[j].bits[0] = mbf_zero.bits[0];
    involved[j].bits[1] = mbf_zero.bits[1];
    involved[j].bits[2] = mbf_zero.bits[2];
    involved[j].bits[3] = mbf_zero.bits[3];
  }
    /*      for (int i = 0; i < num_muni; i++) 
	    if(LOGICAL(involved_sexp)[j + i * num_coal]) involved[j].bits[i / 64] |= one << (i % 64); */
  int prev_rows = 0;
  for(int coal_size=1; coal_size <= LENGTH(involved_sexp); coal_size++) {
    /* SEXP coal_mat = VECTOR_ELT(involved_sexp, coal_size-1); */
    SEXP bm = VECTOR_ELT(involved_sexp, coal_size-1);
    if(!isNull(bm)) {
      /* could we use MatrixAccessor here?  do we have to? */
      const short *bm_coal = (const short *)(((BigMatrix *)R_ExternalPtrAddr(bm))->matrix());
      /* int nrows = LENGTH(coal_mat) / coal_size; */
      int nrows = ((BigMatrix *)R_ExternalPtrAddr(bm))->nrow();
      for(int i=0; i < coal_size; i++) {
	for(int j=0; j < nrows; j++) {
	  /*	int muni = INTEGER(coal_mat)[j + i * nrows]; */
	  short muni = bm_coal[j + i * nrows];
	  involved[j + prev_rows].bits[muni / 64] |= one << (muni % 64);
	}
      }
      prev_rows += nrows;
    }
  }
}

extern "C"
SEXP get_all_refinements(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp) {
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  int *cur_refs = (int *)malloc(sizeof(int) * num_coal);
  prep_involved(involved_sexp, involved, num_coal, num_muni);
  SEXP ret;
  PROTECT(ret = NEW_LIST(num_coal));
  for(int i = 0; i < num_coal; i++) {
    int num_refs = 0;
    for(int j = 0; j < num_coal; j++) 
      if(!mbf_any(mbf_and(involved[j], mbf_not(involved[i])))) /* then j is a refinement, so add it */
	if(j != i) cur_refs[num_refs++] = j; /* don't add i as a refinement of i */
    SEXP elt = NEW_INTEGER(num_refs);
    for(int e = 0; e < num_refs; e++)
      INTEGER(elt)[e] = cur_refs[e];
    SET_VECTOR_ELT(ret, i, elt);
  }
  /*  elt = NEW_INTEGER(1);
  INTEGER(elt)[0] = num_refs;
  SET_VECTOR_ELT(ret, 1, elt); */
  UNPROTECT(1);
  free(involved);
  free(cur_refs);
  return(ret);
}

extern "C"
SEXP do_parts(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP coal_l, SEXP burn_in_reps_sexp, SEXP total_reps_sexp, SEXP num_m, SEXP stop_after_sexp, SEXP rand_sexp) {
  SEXP ret;
  PROTECT(ret = NEW_LIST(3));
  igraph_vector_long_t cl;
  igraph_vector_long_t cl_orig;
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  int total_reps = asInteger(total_reps_sexp);
  int burn_in_reps = asInteger(burn_in_reps_sexp);
  int stop_after = asInteger(stop_after_sexp); /* should we stop after a fixed number of changes? */
  double total_choices = 1; /* an estimate of how many coalitions could have been reached - becomes inaccurate very quickly... */
  //  int coarsening[num_coal];
  char *feasible_c = (char *)malloc(num_coal);
  int *coarsening = (int *)malloc(sizeof(int) * num_coal);
  // muni_bitfield involved[num_coal];
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  // igraph_vector_ptr_t *refmnt[num_coal];
  igraph_vector_ptr_t **refmnt = (igraph_vector_ptr_t **)malloc(sizeof(igraph_vector_ptr_t *) * num_coal);
  for(int i = 0; i < num_coal; i++)
    refmnt[i] = NULL;
  double K_max = num_coal; /* a first (lower bound) guess at the maximum number of transitions out of a state */
  unsigned long long one = 1;
  igraph_vector_long_init(&cl,0);
  for(int i = 0; i < length(coal_l); i++) /* convert from one-based to zero-based indexing */
    igraph_vector_long_push_back(&cl, INTEGER(coal_l)[i] - 1);
  igraph_vector_long_copy(&cl_orig, &cl);
  srand(asInteger(rand_sexp)); /* srand(25) etc. to initialize to something replicable... */
  
  prep_involved(involved_sexp, involved, num_coal, num_muni);

  muni_bitfield unused = {{0, 0, 0, 0}}; 
  for (int m = num_muni; m < 256; m++) {
    unused.bits[m / 64] |= one << (m % 64); 
  }

  int num_coarsenings = 0;
  double num_refinements = 0;
  unsigned char changed = 1; /* do we need to redo coarsenings etc? */
  for(int repnum = 0; repnum < total_reps; repnum++) {
    if(changed) { /* build the list of coarsenings */
      for(int check_coal = 0; check_coal < num_coal; check_coal++) {
	coarsening[check_coal] = 1; /* and then check to rule this out */
	for (int i = 0; i < igraph_vector_long_size(&cl); i++)
	  if (mbf_any(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]])))
	    if(!equal(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]]), involved[VECTOR(cl)[i]]))
	      coarsening[check_coal] = 0;
      }
      num_coarsenings = 0;
      for(int check_coal = 0; check_coal < num_coal; check_coal++)
	num_coarsenings += coarsening[check_coal];
    
    /* build the list of refinements */
    for(int iter = 0; iter < igraph_vector_long_size(&cl); iter++)
      if(refmnt[VECTOR(cl)[iter]] == NULL) {
      muni_bitfield avail = {{0, 0, 0, 0}}; 
      for (int i = 0; i < 4; i++)
	avail.bits[i] = involved[VECTOR(cl)[iter]].bits[i];

      for (int m = num_muni; m < 256; m++)
	avail.bits[m / 64] |= one << (m % 64); 

      /* now mark as feasible only those coalitions for which all the required munis are available */
      for (int i = 0; i < num_coal; i++)
	feasible_c[i] = !(mbf_any(mbf_and(mbf_not(avail), involved[i])));

      /*  Go through all the feasible coalitions, and get rid of the ones that are no longer feasible */
      refmnt[VECTOR(cl)[iter]] = refinement(feasible_c, involved, avail, unused, num_coal, INTEGER(num_m), VECTOR(cl)[iter]);
      }

    num_refinements = 0;
    for(int i = 0; i < igraph_vector_long_size(&cl); i++) /* need to divide because we cut some refinements */
      num_refinements += (double)igraph_vector_ptr_size(refmnt[VECTOR(cl)[i]]) / *cp_end[INTEGER(num_m)[VECTOR(cl)[i]]-1];
    }
    if(stop_after < 0) total_choices = num_coarsenings + num_refinements; /* i.e. "final" rather than "total" */ 
    double rnd = ((double)rand()) / (double)RAND_MAX;


    if(repnum < burn_in_reps && K_max < num_coarsenings + num_refinements)  /* update */
      K_max = num_coarsenings + num_refinements;
    rnd *= K_max;
    if(rnd < num_coarsenings + num_refinements) { /* we have a transition */
      changed = 1;
      if(stop_after >= 0) total_choices *= num_coarsenings + num_refinements;
      int selected = 0;
      int selected_c = -1;
      for(int c = 0; c < num_coal; c++)
	if(coarsening[c]) {    /* choose a coarsening */
	  if(rnd < 1) {
	    selected = c;
	    break;
	  } else rnd--;
	}
      if(selected == 0) { /* looks like it's going to be a refinement... */
	//  rnd = ((double)rand()) / (double)RAND_MAX;
	//  rnd *= num_refinements;
	for(int i = 0; i < igraph_vector_long_size(&cl); i++)
	  if(rnd < igraph_vector_ptr_size(refmnt[VECTOR(cl)[i]]) / *cp_end[INTEGER(num_m)[VECTOR(cl)[i]]-1]) {
	    selected = i;
	    selected_c = (int)floor(rnd * *cp_end[INTEGER(num_m)[VECTOR(cl)[i]]-1]);
	    break;
	  } else rnd -= igraph_vector_ptr_size(refmnt[VECTOR(cl)[i]]) / *cp_end[INTEGER(num_m)[VECTOR(cl)[i]]-1];
      }
      /* but now what coalitions does this replace? */
      igraph_vector_long_update(&cl_orig, &cl);
      if(selected_c == -1) { /* must be a coarsening */
	/* need to add something here to take into account the fact that nothing might be selected... */
	igraph_vector_long_clear(&cl);
	igraph_vector_long_push_back(&cl, selected);
	for(int i = 0; i < igraph_vector_long_size(&cl_orig); i++)  /* let through only the ones that aren't in coarsening */
	  if(!mbf_any(mbf_and(involved[VECTOR(cl_orig)[i]], involved[selected]))) {
	    igraph_vector_long_push_back(&cl, VECTOR(cl_orig)[i]);
	  } else { /* free the memory where refinements were stored, to avoid ever-increasing memory footprint */
	    for(int j = 0; j < igraph_vector_ptr_size(refmnt[VECTOR(cl_orig)[i]]); j++) {
	      igraph_vector_long_destroy((igraph_vector_long_t*)igraph_vector_ptr_e(refmnt[VECTOR(cl_orig)[i]], j));
	      free(igraph_vector_ptr_e(refmnt[VECTOR(cl_orig)[i]], j));
	    }
	    igraph_vector_ptr_destroy(refmnt[VECTOR(cl_orig)[i]]);
	    free(refmnt[VECTOR(cl_orig)[i]]);
	    refmnt[VECTOR(cl_orig)[i]] = NULL;
	  }
      } else { /* a refinement */
	igraph_vector_long_append(&cl, (igraph_vector_long_t *)igraph_vector_ptr_e(refmnt[VECTOR(cl_orig)[selected]], selected_c));

	for(int j = 0; j < igraph_vector_ptr_size(refmnt[VECTOR(cl)[selected]]); j++) {
	  igraph_vector_long_destroy((igraph_vector_long_t*)igraph_vector_ptr_e(refmnt[VECTOR(cl)[selected]], j));
	  free(igraph_vector_ptr_e(refmnt[VECTOR(cl)[selected]], j));
	}
	igraph_vector_ptr_destroy(refmnt[VECTOR(cl)[selected]]);
	free(refmnt[VECTOR(cl)[selected]]);
	refmnt[VECTOR(cl)[selected]] = NULL;

	igraph_vector_long_remove(&cl, selected);
      }
    } else changed = 0;
    if(changed == 1) {
      if(stop_after > 0) stop_after--;
      if(stop_after == 0) break;
    }
  }
  /* prepare output */
  /*  PROTECT(ret = NEW_LIST(igraph_vector_ptr_size(refmnt[igraph_vector_long_tail(&cl)])));
  for(int c = 0; c < length(ret); c++) {
    SEXP tmp_n = NEW_INTEGER(igraph_vector_long_size(igraph_vector_ptr_e(refmnt[igraph_vector_long_tail(&cl)], c)));
    for(int i = 0; i < length(tmp_n); i++)
            INTEGER(tmp_n)[i] = igraph_vector_long_e(igraph_vector_ptr_e(refmnt[igraph_vector_long_tail(&cl)], c), i); 
    SET_VECTOR_ELT(ret, c, tmp_n);
    } */
  SEXP cl_sexp = NEW_INTEGER(igraph_vector_long_size(&cl));
  for(int i = 0; i < length(cl_sexp); i++)
    INTEGER(cl_sexp)[i] = igraph_vector_long_e(&cl, i); 
  SET_VECTOR_ELT(ret, 1, cl_sexp);
  SEXP cl_orig_sexp = NEW_INTEGER(igraph_vector_long_size(&cl_orig));
  for(int i = 0; i < length(cl_orig_sexp); i++)
    INTEGER(cl_orig_sexp)[i] = igraph_vector_long_e(&cl_orig, i); 
  SET_VECTOR_ELT(ret, 0, cl_orig_sexp);
  SEXP total_choices_sexp = NEW_NUMERIC(1);
  REAL(total_choices_sexp)[0] = total_choices;
  SET_VECTOR_ELT(ret, 2, total_choices_sexp);
  UNPROTECT(1);
  free(feasible_c);
  free(coarsening); 
  free(involved);


  for(int i = 0; i < num_coal; i++)
    if(refmnt[i] != NULL) { /* then we need to free the memory somehow... */
      for(int j = 0; j < igraph_vector_ptr_size(refmnt[i]); j++) {
	igraph_vector_long_destroy((igraph_vector_long_t*)igraph_vector_ptr_e(refmnt[i], j));
	free(igraph_vector_ptr_e(refmnt[i], j));
      }
      igraph_vector_ptr_destroy(refmnt[i]);
      free(refmnt[i]);
      refmnt[i] = NULL;
    }
  free(refmnt);
  return(ret);
}

/* for a given coalition out of a given set of coalitions, calculate all feasible refinements */
/* need to modify this to only calculate a random subset of all feasible refinements */
igraph_vector_ptr_t *refinement(char *feasible_c, muni_bitfield *involved, muni_bitfield avail, muni_bitfield unused, int num_coal, int *num_m, int initial_coal) {
  double *cp[] = {cp1, cp2, cp3, cp4, cp5, cp6, cp7, cp8, cp9, cp10, cp11, cp12, cp13, cp14,cp15};
  int size = num_m[initial_coal];
  double *ncut_cum = cp[size-1];
  int depth = 0;
  int depth_munis = 0;
  int total_weight = 1;
  long coal = 0;
  /*  long ncut_cum[] = {0.5, 0.25, 0.125, 0.0625, 0.01, 0.001, 0.0001, 0.00001,
		     0.000001, 0.0000001, 0.00000001, 0.000000001, 0.0000000001,
		     0.00000000001, 0.000000000001}; */
  /*  double ncut_cum[] = {1, 1, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25,
		     0.25, 0.25, 0.25, 0.25, 0.25,
		     0.25, 0.25}; */
  //  int partnum = 0;
  igraph_vector_long_t *current = (igraph_vector_long_t *)malloc(sizeof(igraph_vector_long_t));
  igraph_vector_long_init(current, 0);
  igraph_stack_long_t elim_c[MAX_DEPTH];
  igraph_vector_ptr_t *parts = (igraph_vector_ptr_t *)malloc(sizeof(igraph_vector_ptr_t));
  igraph_vector_ptr_init(parts, 0);
  for(int i = 0; i < MAX_DEPTH; i++) {
    igraph_stack_long_init(&(elim_c[i]), 0);
    igraph_stack_long_empty(&elim_c[i]);
  }

  while(coal < num_coal || !igraph_vector_long_empty(current)) {
    total_weight = 0;
    //    srand(13);
    double rnd = ((double)rand()) / (double)RAND_MAX;
    /* find the next valid coalition */
        while (coal < num_coal && (!feasible_c[coal] || rnd > ncut_cum[depth_munis + num_m[coal]] / ncut_cum[depth_munis])) { 
	  //  while (coal < num_coal && !feasible_c[coal]) {
	coal++; 
	rnd = ((double)rand()) / (double)RAND_MAX; /* not taking the next one should be independent of not taking this one */
      }
    if(coal < num_coal) { /* did we find a coalition? */
      igraph_vector_long_push_back(current, coal);
      avail = mbf_and(avail, mbf_not(involved[coal]));
      for (int i = coal; i < num_coal; i++)  {
	if(feasible_c[i])
	    if(mbf_any(mbf_and(involved[i], involved[coal]))) {
	      feasible_c[i] = 0;
	      igraph_stack_long_push(&elim_c[depth], i); /* eliminated now, but could be revived later... */
	  }
	if(feasible_c[i]) total_weight++; /* note that this now takes into account the above changes */
      }
      depth++;
      depth_munis += num_m[coal];
    }

    if((!total_weight) && !igraph_vector_long_empty(current)) { /* nothing left to do here - did we get everything? */
      char is_valid = 1;
      if(mbf_any(mbf_and(avail, mbf_not(unused)))) is_valid = 0;
      if(is_valid) { /* store it as a valid partition */
	igraph_vector_long_t *old = current;
	current = (igraph_vector_long_t *)malloc(sizeof *current);
	igraph_vector_ptr_push_back(parts, old);
	igraph_vector_long_copy(current, old);
      } 
      coal = igraph_vector_long_pop_back(current); 
      avail = mbf_or(avail, involved[coal]);
      depth--;
      depth_munis -= num_m[coal];
      coal++; /* don't allow the same coalition again... */
      while(!igraph_stack_long_empty(&elim_c[depth])) 
	feasible_c[igraph_stack_long_pop(&elim_c[depth])] = 1;
    }
  }
  for(int i = 0; i < MAX_DEPTH; i++)
      igraph_stack_long_destroy(&elim_c[i]);
  return(parts);
}

extern "C"
SEXP get_part_deviations(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP coal_l) {
  igraph_vector_long_t cl;
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  char *is_coarsening = (char *)malloc(sizeof(char) * num_coal);
  char *is_refinement = (char *)malloc(sizeof(char) * num_coal);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  /* igraph_vector_ptr_t *cur_refmnt; */
  unsigned long long one = 1;
  SEXP ret = NEW_LOGICAL(num_coal);
  PROTECT(ret);
  igraph_vector_long_init(&cl,0);
  for(int i = 0; i < length(coal_l); i++) /* convert from one-based to zero-based indexing */
    igraph_vector_long_push_back(&cl, INTEGER(coal_l)[i] - 1);

  prep_involved(involved_sexp, involved, num_coal, num_muni);

  muni_bitfield unused = {{0, 0, 0, 0}}; 
  for (int m = num_muni; m < 256; m++) {
    unused.bits[m / 64] |= one << (m % 64); 
  }

  for(int check_coal = 0; check_coal < num_coal; check_coal++) {
    is_coarsening[check_coal] = 1; /* and then check to rule this out */
    for (int i = 0; i < igraph_vector_long_size(&cl); i++)
      if (mbf_any(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]])))
	if(!equal(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]]), involved[VECTOR(cl)[i]]))
	  is_coarsening[check_coal] = 0;
  }

  for(int check_coal = 0; check_coal < num_coal; check_coal++) {
    is_refinement[check_coal] = 0;
    for (int i = 0; i < igraph_vector_long_size(&cl); i++)
      if(equal(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]]), involved[check_coal]))
	  is_refinement[check_coal] = 1;
  }

  for(int i = 0; i < num_coal; i++) {
    if(is_coarsening[i] || is_refinement[i]) LOGICAL(ret)[i] = 1;
    else LOGICAL(ret)[i] = 0;
  }
  
  free(is_coarsening);
  free(is_refinement);
  free(involved);
  UNPROTECT(1);
  return(ret);
}


extern "C"
SEXP get_m_deviations(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP coal_l, SEXP fname_sexp) {
  igraph_vector_long_t cl;
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  short actual_len = length(coal_l);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  /* igraph_vector_ptr_t *cur_refmnt; */
  /* unsigned long long one = 1; */
  SEXP ret = NEW_INTEGER(length(fname_sexp));
  PROTECT(ret);
  FILE *out_file[15];
  for(short i = 0; i < length(fname_sexp); i++) {
    out_file[i] = fopen(CHAR(STRING_ELT(fname_sexp, i)), "w");
    INTEGER(ret)[i] = 0;
  }

  igraph_vector_long_init(&cl, 0);
  for(int i = 0; i < actual_len; i++) /* convert from one-based to zero-based indexing */
    igraph_vector_long_push_back(&cl, INTEGER(coal_l)[i] - 1);

  prep_involved(involved_sexp, involved, num_coal, num_muni);

  for(int check_coal = 0; check_coal < num_coal; check_coal++) {
    int perp[actual_len]; /* actual mergers containing perpetrators of the deviation */
    short sum_perp; 
    for(int i = 0; i < actual_len; i++)
      perp[i] = 0;
    sum_perp = 0;
    for(int m = 0; m < actual_len; m++)
      if(mbf_any(mbf_and(involved[check_coal], involved[VECTOR(cl)[m]])))
	if(!equal(involved[check_coal], involved[VECTOR(cl)[m]]))
	  perp[sum_perp++] = m;
    if(sum_perp > 0) {
      fprintf(out_file[sum_perp-1], "%d,", check_coal);
      for(int i=0; i<sum_perp-1; i++)
	fprintf(out_file[sum_perp-1], "%d,", perp[i]);  
      fprintf(out_file[sum_perp-1], "%d\n", perp[sum_perp-1]);
      INTEGER(ret)[sum_perp-1]++;
    }
  }
  for(short i = 0; i < length(fname_sexp); i++)
    fclose(out_file[i]);
  UNPROTECT(1);
  return(ret);
}


//extern "C"
//SEXP no_ref_dev(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP u_sexp) {
//  double *u = REAL(u_sexp);
//  int num_coal = asInteger(num_coal_sexp);
//  int num_muni = asInteger(num_muni_sexp);
//  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
//  SEXP ret = NEW_LOGICAL(num_coal);
//  PROTECT(ret);
//  /*  vmlSetMode( VML_HA | VML_DOUBLE_CONSISTENT | VML_ERRMODE_IGNORE ); */
//  prep_involved(involved_sexp, involved, num_coal, num_muni);
//  LOGICAL(ret)[0] = 1;
//  for(int i=1; i < num_coal; i++) {
//    short dev = 0;
//    for(int j=0; j < i; j++) {
//      if(equal(mbf_and(involved[i], involved[j]), involved[j])) { /* it's a refinement */
//	dev = 1;
//	for(short k=0; k < num_muni; k++) {
//	  if(u[i + k*num_coal] > u[j + k*num_coal]) {
//	    dev = 0;
//	    break;
//	  }
//	}
//      }
//      if(dev) break;
//    }
//    if(dev) LOGICAL(ret)[i] = 0;
//    else LOGICAL(ret)[i] = 1;
//  }
//  UNPROTECT(1);
//  return(ret);
//}

extern "C"
SEXP no_ref(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP u_sexp, SEXP nc_sexp) {
  int *nc = INTEGER(nc_sexp);
  double *u[LENGTH(u_sexp)];
  for(int i=0; i < LENGTH(u_sexp); i++) {
    if(nc[i] > 0) u[i] = REAL(VECTOR_ELT(u_sexp, i));  /* otherwise leave it uninitialized */
  }
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  SEXP ret = NEW_LOGICAL(num_coal);
  PROTECT(ret);
  /*  vmlSetMode( VML_HA | VML_DOUBLE_CONSISTENT | VML_ERRMODE_IGNORE ); */
     prep_involved(involved_sexp, involved, num_coal, num_muni);
  LOGICAL(ret)[0] = 1;
  short size = 1;
  for(int i=1, size_count=1; i < num_coal; i++, size_count++) {
    short dev = 0;
    if(size_count==nc[size-1]) {
      
      /*	size_count=0;
  		size++; */
      size_count=-1;
      i--;
      size++;
      continue;
    }
    short size_ref = 1;
      for(int j=0, size_count_ref=0; j < i; j++, size_count_ref++) {
  	if(size_count_ref==nc[size_ref-1]) {
  	  /*size_count_ref=0;
  	    size_ref++; */
  	  size_count_ref=-1;
  	  j--;
  	  size_ref++;
  	  continue;
  	}
      if(equal(mbf_and(involved[i], involved[j]), involved[j])) { /* it's a refinement */
  	dev = 1;
  	short m_idx = 0;
  	short m_idx_ref = 0;
  	for(short k=0; k < num_muni; k++) {
  	  if(mbf_any(mbf_and(involved[i], involved[k]))) { /* this municipality is in the coal we're checking */
  	    if(mbf_any(mbf_and(involved[j], involved[k]))) { /* is also in the refinement we're checking... */
  	      if(u[size-1][size_count + m_idx*nc[size-1]] > u[size_ref-1][size_count_ref + m_idx_ref*nc[size_ref-1]]) {
  		dev = 0;
  		break;
  	      }
  	      m_idx_ref++;
  	    }
  	    m_idx++;
  	  }
  	}
  	}
      if(dev) break;
    }
    if(dev) LOGICAL(ret)[i] = 0;
    else LOGICAL(ret)[i] = 1;
  }
  UNPROTECT(1);
  return(ret);
}

extern "C"
SEXP rnd_stable_part(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP exclude_sexp, SEXP coal_l, SEXP rand_sexp) {
  int num_coarsening;
  igraph_vector_long_t cl;
  igraph_vector_long_t cl_orig;
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  char is_coarsening[num_coal];
  for (int i = 0; i < num_coal; i++) {
    if(INTEGER(exclude_sexp)[i]) is_coarsening[i] = 0;
    else is_coarsening[i] = 1;
  }
  igraph_vector_long_init(&cl,0);
  for(int i = 0; i < length(coal_l); i++) /* convert from one-based to zero-based indexing */
    igraph_vector_long_push_back(&cl, INTEGER(coal_l)[i] - 1);
  igraph_vector_long_init(&cl_orig,0);

  prep_involved(involved_sexp, involved, num_coal, num_muni);
  srand(asInteger(rand_sexp)); /* srand(25) etc. to initialize to something replicable... */
  do {
    num_coarsening = 0;
    for(int check_coal = 0; check_coal < num_coal; check_coal++) {
      if(is_coarsening[check_coal]) {
	for (int i = 0; i < igraph_vector_long_size(&cl); i++)
	  if (mbf_any(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]]))) {
	    if(!equal(mbf_and(involved[check_coal], involved[VECTOR(cl)[i]]), involved[VECTOR(cl)[i]]))
	      is_coarsening[check_coal] = 0;
	    if(equal(involved[check_coal], involved[VECTOR(cl)[i]]))
	      is_coarsening[check_coal] = 0;
	  }
      }
      if(is_coarsening[check_coal]) num_coarsening++;
    }
    
    if(num_coarsening>0) {
      double rnd = ((double)num_coarsening) * ((double)rand()) / (double)RAND_MAX;
      int selected_to_go = (int)floor(rnd);
      int selected = -1;
      for(int check_coal = 0; check_coal < num_coal; check_coal++) {
	if(is_coarsening[check_coal])
	  if(selected_to_go--==0) {
	    selected = check_coal;
	    break;
	  }
      }
      igraph_vector_long_update(&cl_orig, &cl);
      igraph_vector_long_clear(&cl);
      igraph_vector_long_push_back(&cl, selected);
      for(int i = 0; i < igraph_vector_long_size(&cl_orig); i++)  /* let through only the ones that aren't in coarsening */
	if(!mbf_any(mbf_and(involved[VECTOR(cl_orig)[i]], involved[selected])))
	  igraph_vector_long_push_back(&cl, VECTOR(cl_orig)[i]);
    }
  } while(num_coarsening > 0);

  SEXP ret = NEW_INTEGER(igraph_vector_long_size(&cl));
  //SEXP ret = NEW_INTEGER(1);
  PROTECT(ret);
  for(int i = 0; i < length(ret); i++)
    INTEGER(ret)[i] = igraph_vector_long_e(&cl, i) + 1; /* switch back to 1-based indexing */
  //INTEGER(ret)[0] = num_coarsening;
  UNPROTECT(1);
  free(involved);
  igraph_vector_long_destroy(&cl);
  igraph_vector_long_destroy(&cl_orig);
  return(ret);
}

extern "C"
SEXP stable_part_m(SEXP involved_sexp, SEXP num_coal_sexp, SEXP num_muni_sexp, SEXP u_sexp) {
  igraph_vector_long_t cl;
  igraph_vector_long_init(&cl,0);
  int num_coal = asInteger(num_coal_sexp);
  int num_muni = asInteger(num_muni_sexp);
  double *u = REAL(u_sexp);
  muni_bitfield *involved = (muni_bitfield *)malloc(sizeof(muni_bitfield) * num_coal);
  prep_involved(involved_sexp, involved, num_coal, num_muni);
  muni_bitfield available = involved[0];
  for(int i = 1; i < num_muni; i++)
    available = mbf_or(available, involved[i]); /* assumption here is that singleton coalitions are always feasible, and always first */

  while(mbf_any(available)) {
    double max_u = NEGINF;
    int max_u_idx = -1;
    for(int check_coal = 0; check_coal < num_coal; check_coal++)
      if(u[check_coal] > max_u && equal(mbf_and(available, involved[check_coal]), involved[check_coal])) {
	max_u = u[check_coal];
	max_u_idx = check_coal;
      }
    igraph_vector_long_push_back(&cl, max_u_idx);
    available = mbf_and(available, mbf_not(involved[max_u_idx]));
  }    
  SEXP ret = NEW_INTEGER(igraph_vector_long_size(&cl));
  PROTECT(ret);
  for(int i = 0; i < length(ret); i++)
    INTEGER(ret)[i] = igraph_vector_long_e(&cl, i) + 1; /* switch to 1-based indexing */
  UNPROTECT(1);
  free(involved);
  igraph_vector_long_destroy(&cl);
  return(ret);
}

/* icc -I/bbkinghome/weese/include/igraph -I/bbkinghome/weese/lib64/R/include -L/bbkinghome/weese/lib/ -Wl,--rpath -Wl,/bbkinghome/weese/lib parts4.c -o parts.so -shared -fpic -lguide -pthread -ligraph */
/* gcc --std=c99 -Wall -I/home2/egw5/src/igraph-0.5.3/include -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/bigmemory -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/ -I/usr/local/cluster/hpc/Applications/R/R-2.10.1/lib64/R/include -L/home2/egw5/lib/ -Wl,--rpath -Wl,/home2/egw5/lib parts.c -o parts.so -shared -fpic -pthread -ligraph */
/* gcc --std=c99 -Wall -I/bbkinghome/weese/include/igraph -I/bbkinghome/weese/lib64/R/include -L/bbkinghome/weese/lib/ -Wl,--rpath -Wl,/bbkinghome/weese/lib parts4.c -o parts.so -shared -fpic -pthread -ligraph */
/* g++ -Wall -I/home2/egw5/src/igraph-0.5.3/include -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/bigmemory -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/ -I/usr/local/cluster/hpc/Applications/R/R-2.10.1/lib64/R/include -L/home2/egw5/lib/ -Wl,--rpath -Wl,/home2/egw5/lib -O1 parts.c -o parts.so -shared -fpic -pthread -ligraph */
/* icpc -I/home2/egw5/src/igraph-0.5.3/include -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/bigmemory -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.10/bigmemory/include/ -I/usr/local/cluster/hpc/Applications/R/R-2.10.1/lib64/R/include -L/home2/egw5/lib/ -Wl,--rpath -Wl,/home2/egw5/lib -O3 parts.C -o parts.so -shared -fpic -pthread -ligraph */
/* g++ -Wall -I/home2/egw5/src/igraph-0.6/include -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.14/bigmemory/include/bigmemory -I/home2/egw5/R/x86_64-unknown-linux-gnu-library/2.14/bigmemory/include/ -I/usr/local/cluster/hpc/Software/BDK/Applications/R/R-2.14.0-gcc/lib64/R/include -L/home2/egw5/lib/ -Wl,--rpath -Wl,/home2/egw5/lib -O3 parts.cpp -o parts.so -shared -fpic -pthread -ligraph */

/* icc seems to take maybe 2/3rds the time of gcc, so it's probably worth using... */

