## echo "Rscript --verbose /home2/egw5/japan_restud/4_pgraph.R" | qsub -q fas_devel
## takes less than 1 min on 1 cpu

options(echo=TRUE)
proj.home <- "/home2/egw5/japan_restud/"

load(paste(proj.home, "gappei.RData", sep=""))
final.mergers <- !rev(duplicated(rev(gappei$merger.name)))  #use only the most recent mergers
mergers.l <- gappei$merger[final.mergers]
names(mergers.l) <- gappei$merger.name[final.mergers]
stopifnot(length(unlist(mergers.l)) == length(unique(unlist(mergers.l))))  #point was to eliminate any duplicates

load(paste(proj.home, "shp.RData", sep=""))
library("spdep")
tmp <- poly2nb(shp, queen=FALSE) #queen doesn't change number of disconnected mergers (34 total)
adj.mat <- nb2mat(tmp, style="B", zero.policy=TRUE)
colnames(adj.mat) <- row.names(adj.mat)

library("igraph") #ONE-BASED INDEXING
b.graph <- graph.adjacency(adj.mat, mode="undirected", diag=FALSE)
#V(b.graph)$label <- row.names(adj.mat)

load(paste(proj.home, "mesh.RData", sep=""))
tmp <- t(sapply(cells, get.med))
V(b.graph)$lon <- tmp[V(b.graph)$name, 1]
V(b.graph)$lat <- tmp[V(b.graph)$name, 2]

E(b.graph)$sea <- FALSE

# ritou that I connected when I could have avoided it:
# 1 in hokkaidou, 1 in niigata, 1 in "30", 1 in "32" (at 50km), 1 in "37", 1 in "42", 2 in "43", 1 in "44", 0 in "46" (5 in "47")

# first, let's add connections between islands and the mainland when those mergers actually did occur:
b.graph <- add.edges(b.graph, c(V(b.graph)["17405"], V(b.graph)["17202"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["32308"], V(b.graph)["32303"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["34206"], V(b.graph)["34444"]))
#b.graph <- add.edges(b.graph, c(V(b.graph)["34430"], V(b.graph)["34206"])) #already linked?
b.graph <- add.edges(b.graph, c(V(b.graph)["34481"], V(b.graph)["34482"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["34327"], V(b.graph)["34323"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["34314"], V(b.graph)["34424"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["34313"], V(b.graph)["34424"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38354"], V(b.graph)["38343"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38346"], V(b.graph)["38343"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38363"], V(b.graph)["38211"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["38355"], V(b.graph)["38343"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["38348"], V(b.graph)["38343"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["40365"], V(b.graph)["40364"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["42386"], V(b.graph)["42208"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42387"], V(b.graph)["42208"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["42384"], V(b.graph)["42389"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["42382"], V(b.graph)["42385"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42381"], V(b.graph)["42385"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["42313"], V(b.graph)["42314"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42312"], V(b.graph)["42311"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42303"], V(b.graph)["42301"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42302"], V(b.graph)["42301"])) #ritou, ok
b.graph <- add.edges(b.graph, c(V(b.graph)["42207"], V(b.graph)["42385"]))

# now, let's add connections even though mergers didn't occur there:
# (ignore tokyo because the entire coastline is special wards...)
b.graph <- add.edges(b.graph, c(V(b.graph)["1367"], V(b.graph)["1366"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["15586"], V(b.graph)["15585"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["28421"], V(b.graph)["28463"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["30427"], V(b.graph)["30425"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["32527"], V(b.graph)["32302"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["37364"], V(b.graph)["37343"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42383"], V(b.graph)["42384"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["43526"], V(b.graph)["43484"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["43521"], V(b.graph)["43321"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["44322"], V(b.graph)["44321"]))
# ignore 30427 because it's landlocked

# now add connections for other mergers:
b.graph <- add.edges(b.graph, c(V(b.graph)["34425"], V(b.graph)["34424"])) 
b.graph <- add.edges(b.graph, c(V(b.graph)["28421"], V(b.graph)["28201"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38351"], V(b.graph)["38350"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38352"], V(b.graph)["38351"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["38349"], V(b.graph)["38350"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["42405"], V(b.graph)["42206"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["43521"], V(b.graph)["43522"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["43526"], V(b.graph)["43527"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["46388"], V(b.graph)["46202"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["46391"], V(b.graph)["46202"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["47374"], V(b.graph)["47206"]))

E(b.graph)[is.na(E(b.graph)$sea)]$sea <- TRUE

E(b.graph)$tobichi <- FALSE
# fix tobichi:  (total of 13, not double-counting the double one below)
b.graph <- add.edges(b.graph, c(V(b.graph)["1669"], V(b.graph)["1666"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["1574"], V(b.graph)["1233"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["1603"], V(b.graph)["1601"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["2385"], V(b.graph)["2382"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["2306"], V(b.graph)["2305"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["2386"], V(b.graph)["2383"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["10307"], V(b.graph)["10203"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["10361"], V(b.graph)["10202"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["15441"], V(b.graph)["15421"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["21523"], V(b.graph)["21214"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["21342"], V(b.graph)["21202"])) #two in one merger here...
b.graph <- add.edges(b.graph, c(V(b.graph)["21384"], V(b.graph)["21202"])) #
b.graph <- add.edges(b.graph, c(V(b.graph)["36481"], V(b.graph)["36485"]))
b.graph <- add.edges(b.graph, c(V(b.graph)["46528"], V(b.graph)["46526"])) # on an island

E(b.graph)[is.na(E(b.graph)$tobichi)]$tobichi <- TRUE
E(b.graph)[is.na(E(b.graph)$sea)]$sea <- FALSE

tmp <- sapply(mergers.l, function(x) is.connected(induced.subgraph(b.graph, which(V(b.graph)$name %in% x))))
stopifnot(all(tmp))  #should have fixed all the problems above

V(b.graph)$label <- substring(V(b.graph)$name, first=nchar(V(b.graph)$name) - 2)

# break down into prefectures
p.graph.l <- list()
for (p in unique(shp@data$pref)) { #already in proper order
  p.graph <- induced.subgraph(b.graph, which(shp@data[V(b.graph)$name, "pref"] == p))
  p.graph.l[[p]] <- p.graph
}
p.graph.l[["長野県"]] <- induced.subgraph(p.graph.l[["長野県"]], V(p.graph.l[["長野県"]])[name!="20431"]) #remove prefecture-switcher
p.graph.l[["岐阜県"]] <- induced.subgraph(b.graph, c(which(shp@data[V(b.graph)$name, "pref"] == "岐阜県"), which(V(b.graph)$name == "20431"))) #add it back in the "right" place
V(p.graph.l[["岐阜県"]])["20431"]$label <- "20431"

for(p in 1:length(p.graph.l)) {
  enames <- rep(NA, ecount(p.graph.l[[p]]))
  for(e in 1:ecount(p.graph.l[[p]]))
    enames[e] <- paste(V(p.graph.l[[p]])[inc(E(p.graph.l[[p]])[e])]$label, collapse="_")
  E(p.graph.l[[p]])$name <- enames
}

save(mergers.l, p.graph.l, file=paste(proj.home, "pgraph.RData", sep=""))
