## ./qc_prep.R
##
## Produces intermediate data files regarding coarsenings and refinements.
## Runs on one size of coalition, one prefecture at a time.
## Submit via PBS job scripts:

# seq 1 100 | perl -ne 'chomp($p=$_); print "Rscript --verbose /home2/egw5/japan_qc/qc_prep.R 1 ", $p, "\n"' > qc_prep_d/qc_prep_1.task
# sqCreateScript -N qc_prep_1 -w 168:00:00 -n 4 -q fas_normal qc_prep_d/qc_prep_1.task | qsub

proj.home <- "/home2/egw5/japan_restud/"
proj.tmp <- "/home2/egw5/scratch/japan_restud/"

options(echo=TRUE)
tmp <- commandArgs(trailingOnly=TRUE)
tmp
p <- as.numeric(tmp[1])
p
boot <- as.numeric(tmp[2])
boot

pnum <- p

if(is.na(boot)) SIZE.MAX <- 16

if(!is.na(boot)) SIZE.MAX <- { # same as rnd.R that generated the bootstrap bits
  if(p == 1) 11
  else if(p == 2) 13
  else if(p == 5) 12
  else if(p == 11) 12
  else if(p == 15) 11
  else if(p == 19) 15 #?
  else if(p == 40) 12
  else if(p == 43) 14
  else if(p == 46) 11
  else if(p == 47) 15
  else 16
}

fname <- paste(proj.home, "pgraph.RData", sep="")
load(fname)
fname <- paste(proj.home, "gappei.RData", sep="")
load(fname)
fname <- paste(proj.home, "fin_h13.RData", sep="")
load(fname)
rownames(df.h13) <- as.character(as.numeric(rownames(df.h13)))
rownames(num.units) <- as.character(as.numeric(rownames(num.units)))

fname <- paste(proj.home, "mesh.RData", sep="")
load(fname)
rm(fname)


fname <- paste(proj.home, "hosei.R", sep="")
source(fname)

fname <- paste(proj.home, "toku_inv.R", sep="")
source(fname)

library("igraph0")  #ZERO-BASED INDEXING
p.graph <- p.graph.l[[p]]

library("bigmemory")
options(bigmemory.typecast.warning=FALSE)

coal.l.tmp <- list()
for (size in 1:SIZE.MAX) {
  if(size==1) {
    coal.l.tmp[[size]] <- as.big.matrix(matrix((1:vcount(p.graph)) - 1, ncol=1), type="short", shared=FALSE)
  } else if (size==2) {
    coal.l.tmp[[size]] <- as.big.matrix(t(sapply(as.list(0:(ecount(p.graph)-1)), function(x) get.edge(p.graph, x))), type="short", shared=FALSE)
  } else {
    fname <- paste(proj.tmp, pnum, "/", size, "_long.csv", sep="")
    print(fname)
    f.bytes <- file.info(fname)[,"size"]
    if(is.na(f.bytes)) stop("missing file")
    coal.l.tmp[[size]] <- if(f.bytes == 0) matrix(NA, nrow=0, ncol=size) else read.big.matrix(fname, type="short", shared=FALSE)
  }
}

max.size <- length(coal.l.tmp)
use.size <- !sapply(coal.l.tmp, is.null)
coal.len <- sapply(coal.l.tmp, function(x) if(is.null(x)) 0 else nrow(x)) #Note:  this could break if there were a hole in the middle

library("orloca")
library("sp")

fname <- paste(proj.home, "calc_dat.R", sep="")
source(fname)


actual.tmp <- rep(NA, vcount(p.graph))
if(is.na(boot)) {  # use the real mergers
  in.p <- substring(names(mergers.l), first=1, last=3) == substring(names(p.graph.l)[p], first=1, last=3)
  p.mergers.l <- mergers.l[in.p]
} else {  # then we're doing a bootstrap... 
  fname <- paste(proj.tmp, p, "/boot/", boot, "-1.RData", sep="")
  load(fname)
  p.mergers.l <- ret.l.1
}
if(length(p.mergers.l) > 1)
  for(i in 1:length(p.mergers.l)) {
    merger <- p.mergers.l[[i]]
    merger.igraphn <- which(V(p.graph)$name %in% merger) - 1
    if(is.na(boot)) {
      merger.name <- names(p.mergers.l)[i]
      tmp <- gappei[gappei$merger.name == merger.name, "jis"]
      merger.jis <- tmp[length(tmp)]  #use the last one
    } else merger.jis <- as.character(p * 1000 + 800 + i)
    muni.l <- list()
    dat.l <- list()
    j <- 1
    for (size in 1:(length(merger))) {  #the only one of the same size should be the merger itself...
      selected <- apply(coal.l.tmp[[size]][,,drop=FALSE], 1, function(x) all(x %in% merger.igraphn))
      for(dev in which(selected)) {
        dev.munis <- coal.l.tmp[[size]][dev,]
        names(dev.munis) <- V(p.graph)[dev.munis]$name
      
        dat.l[[j]]<- calc.dat(muni.jis=names(dev.munis))
        ## ok, now I need to replace a bunch of these with the new calculations
        muni.l[[j]] <- coal.l.tmp[[size]][dev,]
        j <- j + 1
      }
    }
    if(is.na(boot)) fname <- paste(proj.tmp, pnum, "/qc/r", merger.jis, "_new.RData", sep="")
    if(!is.na(boot)) fname <- paste(proj.tmp, pnum, "/boot/", boot, "/r", merger.jis, "_new.RData", sep="")
    save(dat.l, muni.l, file=fname)
  }


#now do the coarsenings
if(is.na(boot)) {
  fname <- paste(proj.tmp, p, "/i_actual.RData", sep="")
  load(fname)
} else { # ... then we're doing a bootstrap
  actual <- 1:vcount(p.graph)
  names(actual) <- V(p.graph)$name
  if(length(p.mergers.l) > 0) actual[unlist(p.mergers.l)] <- 999
}
singletons <- actual <= vcount(p.graph)

size <- 2

f <- function(x) all(singletons[x + 1])
selected <- which(apply(coal.l.tmp[[2]][,], 1, f))

for(coarsening in selected) {
  merger <- coal.l.tmp[[size]][coarsening,]
  muni.l <- c(split(merger, actual[merger+1]), list(merger))
  dat.l <- list()

  dat.l[[1]] <- calc.dat(muni.jis=V(p.graph)[muni.l[[1]]]$name)

  dat.l[[2]] <- calc.dat(muni.jis=V(p.graph)[muni.l[[2]]]$name)
  
  dat.l[[3]] <- calc.dat(muni.jis=V(p.graph)[muni.l[[3]]]$name)
  if(is.na(boot)) fname <- paste(proj.tmp, pnum, "/c", size, "_", coarsening, "_new.RData", sep="")
  if(!is.na(boot)) fname <- paste(proj.tmp, pnum, "/boot/", boot, "/c", size, "_", coarsening, "_new.RData", sep="")
  save(dat.l, muni.l, file=fname)
}


