## ./qc_prep_t0.R
##
## Produces intermediate data files for situation at t=0.
## Runs on one size of coalition, one prefecture at a time.
## Submit via PBS job scripts:

# seq 1 47 | perl -ne 'chomp($p=$_); print "Rscript --verbose /home2/egw5/japan_qc/qc_prep.R ", $p, "\n"' > qc_prep_d/qc_prep_t0.task
# sqCreateScript -N qc_prep_t0 -w 4:00:00 -n 4 -q fas_devel qc_prep_t0.task | qsub

proj.home <- "/home2/egw5/japan_restud/"
proj.tmp <- "/home2/egw5/scratch/japan_restud/"

options(echo=TRUE)
tmp <- commandArgs(trailingOnly=TRUE)
tmp
p <- as.numeric(tmp[1])
p

library("igraph") # ONE-BASED INDEXING

fname <- paste(proj.home, "mesh.RData", sep="")
load(fname)
rm(fname)

fname <- paste(proj.home, "pgraph.RData", sep="")
load(fname)
p.graph <- p.graph.l[[p]]

fname <- paste(proj.home, "gappei.RData", sep="")
load(fname)
fname <- paste(proj.home, "fin_h13.RData", sep="")
load(fname)
rownames(df.h13) <- as.character(as.numeric(rownames(df.h13)))
rownames(num.units) <- as.character(as.numeric(rownames(num.units)))

fname <- paste(proj.home, "hosei.R", sep="")
source(fname)

library("orloca")
library("sp")

fname <- paste(proj.home, "calc_dat.R", sep="")
source(fname)

fname <- paste(proj.home, "toku_inv.R", sep="")
source(fname)

singles.jis <- V(p.graph)$name
singles.dat <- lapply(singles.jis, calc.dat, T0=TRUE)

pairs.jis <- matrix(NA, nrow=vcount(p.graph)*(vcount(p.graph)-1)/2, ncol=2)
n <- 0
for(i in 1:(vcount(p.graph)-1))
  for(j in (i+1):vcount(p.graph)) {
    n <- n + 1
    pairs.jis[n,] <- V(p.graph)$name[c(i,j)]
}

library("parallel")
pairs.dat <- lapply(1:nrow(pairs.jis), function(m) calc.dat(pairs.jis[m,], T0=TRUE))

fname <- paste(proj.tmp, p, "/qc/m2_nc.RData", sep="")
save(singles.jis, singles.dat, pairs.jis, pairs.dat, file=fname)
