## ./rf.R
## Produce some reduced-form graphs (Figures 13-16)

proj.home <- "/home/fas/weese/egw5/japan_qe_final/"
proj.tmp <- "/home/fas/weese/egw5/scratch/japan_qe/"

fname <- paste(proj.home, "anal_base_a.R", sep="")
source(fname)

load(paste(proj.home, "gappei.RData", sep=""))

### from japan.R
singleton.dat.l <- list()
for(p in 1:47) {
  fname <- paste(proj.tmp, p, "/i_long_1_m_new.RData", sep="")
  load(fname)
  rownames(m.dat) <- V(p.graph.l[[p]])$name
  singleton.dat.l[[p]] <- m.dat
}
###

pop <- d.h13[,"pop"]
merger <- pop * 0
merger[unlist(gappei[,"merger"])] <- 1
summary(merger)

sfn13.tmp <- unlist(lapply(singleton.dat.l, function(x) x[,"sfn.struct.h13"] + x[,"sfn.subsidy"] + x[,"sfn.idio"]))
sfn.tmp <- unlist(lapply(singleton.dat.l, function(x) x[,"sfn.struct.h20"] + x[,"sfn.subsidy"] + x[,"sfn.idio"]))
sfr.tmp <- unlist(lapply(singleton.dat.l, function(x) x[,"sfr"]))
sfn <- sfn.tmp[names(pop)]
sfn13 <- sfn13.tmp[names(pop)]
sfr <- sfr.tmp[names(pop)]
summary(sfr/sfn)
summary(sfn/sfn13)

summary(merger)
summary(merger[sfr > sfn])

not.ritou <- !(names(pop) %in% ritou)


library("lattice")
library("latticeExtra")

pfname <- paste(proj.home, "rf_pop.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ pop[not.ritou], panel=panel.smoother, scales=list(x=list(log=TRUE)), ylim=c(0,1), xlab="Population", ylab="Fraction participating in mergers")
dev.off()

pfname <- paste(proj.home, "rf_pop_bw.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ pop[not.ritou], panel=panel.smoother, scales=list(x=list(log=TRUE)), ylim=c(0,1), xlab="Population", ylab="Fraction participating in mergers", col="black", col.se="grey20")
dev.off()

pfname <- paste(proj.home, "rf_dsfn.pdf", sep="")
pdf(file=pfname)
xlab <- expression(paste('"Stick" incentive (fraction of ',  tilde(c), ')'))
xyplot(merger[not.ritou] ~ I(sfn[not.ritou]/sfn13[not.ritou] - 1), panel=panel.smoother, xlab=xlab, ylab="Fraction participating in mergers", ylim=c(0,1))
dev.off()

pfname <- paste(proj.home, "rf_dsfn.pdf", sep="")
pdf(file=pfname)
xlab <- expression(paste('"Stick" incentive (fraction of ',  tilde(c), ')'))
xyplot(merger[not.ritou] ~ I(sfn[not.ritou]/sfn13[not.ritou] - 1), panel=panel.smoother, xlab=xlab, ylab="Fraction participating in mergers", ylim=c(0,1))
dev.off()

pfname <- paste(proj.home, "rf_dsfn_bw.pdf", sep="")
pdf(file=pfname)
xlab <- expression(paste('"Stick" incentive (fraction of ',  tilde(c), ')'))
xyplot(merger[not.ritou] ~ I(sfn[not.ritou]/sfn13[not.ritou] - 1), panel=panel.smoother, xlab=xlab, ylab="Fraction participating in mergers", ylim=c(0,1), col="black", col.se="grey20")
dev.off()

pfname <- paste(proj.home, "rf_sfr_pc.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ I(sfr[not.ritou]/pop[not.ritou]), panel=panel.smoother, scales=list(x=list(log=TRUE, equispaced.log=FALSE)), xlab="SFR per capita", ylab="Fraction participating in mergers", ylim=c(0,1))
dev.off()

pfname <- paste(proj.home, "rf_sfr_pc_bw.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ I(sfr[not.ritou]/pop[not.ritou]), panel=panel.smoother, scales=list(x=list(log=TRUE, equispaced.log=FALSE)), xlab="SFR per capita", ylab="Fraction participating in mergers", ylim=c(0,1), col="black", col.se="grey20")
dev.off()

pfname <- paste(proj.home, "rf_sfr_psfn.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ I(sfr[not.ritou]/sfn[not.ritou]), panel=panel.smoother, scales=list(x=list(log=TRUE, equispaced.log=FALSE)), xlab="SFR (fraction of SFN)", ylab="Fraction participating in mergers", ylim=c(0,1))
dev.off()

pfname <- paste(proj.home, "rf_sfr_psfn_bw.pdf", sep="")
pdf(file=pfname)
xyplot(merger[not.ritou] ~ I(sfr[not.ritou]/sfn[not.ritou]), panel=panel.smoother, scales=list(x=list(log=TRUE, equispaced.log=FALSE)), xlab="SFR (fraction of SFN)", ylab="Fraction participating in mergers", ylim=c(0,1), col="black", col.se="grey20")
dev.off()


