## echo "Rscript --verbose /home2/egw5/japan_restud/1_shp.R" | qsub -q fas_devel 
## takes less than 1 min on 1 cpu

options(echo=TRUE)
proj.home <- "/home/fas/weese/egw5/japan_qe_final/"
library("maptools")
#towgs84 copied from <http://svnmirror.osgeo.org/grass/grass/trunk/lib/gis/datumtransform.table>
#unfortunately there doesn't seem to be a stringsAsFactors argument here, so we need to deal with factors for a while...
tmp <- readShapePoly(paste(proj.home, "japan_ver10/japan", sep=""),
                           proj4string=CRS("+proj=longlat +ellps=bessel +towgs84=-147.54,507.26,680.47"))
shp.tmp <- tmp[tmp$JCODE>0 & tmp$JCODE!=1999,]  #remove uninhabited islands (FLAG==1), lakes (FLAG==2), and disputed kuril islands (JCODE==1999)
shp.tmp@data <- within(shp.tmp@data, {
     levels(CITY1) <- iconv(levels(CITY1),from="SHIFT_JIS",to="UTF-8")
     levels(PREF) <- iconv(levels(PREF),from="SHIFT_JIS",to="UTF-8")
     levels(CITY2) <- iconv(levels(CITY2),from="SHIFT_JIS",to="UTF-8")
     levels(TOWN) <- iconv(levels(TOWN),from="SHIFT_JIS",to="UTF-8")
     jis <- JCODE
     is.ward <- !is.na(CITY2)
     jis[is.ward] <- trunc(jis[is.ward] / 100) * 100 + (jis[is.ward] %% 100 > 30) * 30  #need to change the ward codes to the city codes
     pref <- as.character(PREF)
     county <- as.character(TOWN)
     county[is.na(county)] <- ""
     muni <- as.character(CITY1)
     muni[is.ward] <- as.character(CITY2[is.ward])
     ### add sub-prefectures for Tokyo islands that don't have counties (to ensure that all non-cities have a county)
     county[pref=="東京都" & muni %in% c("大島町", "利島村", "新島村", "神津島村")] <- "大島支庁"
     county[pref=="東京都" & muni %in% c("三宅村", "御蔵島村")] <- "三宅支庁"
     county[pref=="東京都" & muni %in% c("八丈町", "青ヶ島村")] <- "八丈支庁"
     county[pref=="東京都" & muni == "小笠原村"] <- "小笠原支庁"　#none of these merged - what happened with other 離島?
     muni[jis==24482] <- "島ヶ原村" #small ヶ is correct in these cases (not clear if this is exactly right)
     muni[jis==29301] <- "月ヶ瀬村" 
     muni[jis==43525] <- "龍ヶ岳町"
     muni[jis==22328] <- "天城湯ヶ島町"
     muni[jis==11226] <- "鳩ヶ谷市"
     muni[jis==11241] <- "鶴ヶ島市"
     muni[jis==20210] <- "駒ヶ根市"
     muni[jis==3381] <- "金ケ崎町" #there are also some cases where the large ケ should be used (")
     muni[jis==8208] <- "龍ケ崎市"
     muni[jis==12224] <- "鎌ケ谷市"
     ### fix typos
     muni[jis==44502] <- "本耶馬渓町"
     #back out name changes that weren't supposed to have happened as of Oct '95
     muni[jis==15323] <- "横越村"
     muni[jis==8449] <- "東村"
     muni[jis==8446] <- "新利根村"
     muni[jis==8447] <- "河内村"
     muni[jis==8542] <- "五霞村"
     muni[jis==8461] <- "出島村"
     muni[jis==10322] <- "倉淵村"
     muni[jis==11243] <- "吉川町"
     county[jis==11243] <- "北葛飾郡"
     jis[jis==11243] <- 11466
     muni[jis==12231] <- "印西町"
     county[jis==12231] <- "印旛郡"
     jis[jis==12231] <- 12327
     muni[jis==1234] <- "広島町"
     county[jis==1234] <- "札幌郡"
     jis[jis==1234] <- 1301
     muni[jis==1235] <- "石狩町"
     county[jis==1235] <- "石狩郡"
     jis[jis==1235] <- 1302
     muni[jis==26211] <- "田辺町"
     county[jis==26211] <- "綴喜郡"
     jis[jis==26211] <- 26342
     rm(is.ward)
   })

library("gpclib")
gpclibPermit()
shp.tmp2 <- unionSpatialPolygons(shp.tmp, shp.tmp@data$jis)

reorder <- order(as.numeric(sapply(slot(shp.tmp2, "polygons"), function(i) slot(i, "ID"))))
shp.m <- shp.tmp2[reorder]

orig.idx <- match(sapply(slot(shp.m, "polygons"), function(i) slot(i, "ID")), shp.tmp@data$jis)
unique.dat <- data.frame(shp.tmp@data[orig.idx, c("jis", "pref", "county", "muni")], row.names="jis")
shp.m.df <- SpatialPolygonsDataFrame(shp.m, data=unique.dat)
shp <- shp.m.df

save(shp, file=paste(proj.home, "shp.RData", sep=""))
