# ./social_planner.R
# Return the optimal population of a municipality, assuming a uniform population distribution

TAX.FLOOR <- TRUE  # Use the parameters from Column I of Table 2
par.best <- c(B2=0.2164504, B0=219.5554103, B1=5.0432098, B4=0.4882909, B3=1.0446544)

proj.home <- "/home/fas/weese/egw5/japan_qe/"
proj.tmp <- "/home/fas/weese/egw5/scratch/japan_qe/"

par.calibrated <- c(B0=NA, B1=NA, B2=NA, B3=1, B4=1)
par.calibrated["tau.bar"] <- 0.084 * 4/3
par.calibrated["r"] <- 0.02
par.calibrated["disc.santei.h13"] <- par.calibrated["r"] * sum(c(1/(1+par.calibrated["r"])^(1:5), c(.9, .7, .5, .3, .1)/(1+par.calibrated["r"])^(6:10)))
par.calibrated["disc.santei.h20"] <- par.calibrated["r"] * sum(c(1/(1+par.calibrated["r"])^(1:10), c(.9, .7, .5, .3, .1)/(1+par.calibrated["r"])^(11:15)))
par.calibrated["disc.tokurei"] <- par.calibrated["r"] * sum(1/(1+par.calibrated["r"])^(1:10)) * 0.95 * 0.70 * (sum(1/(1+par.calibrated["r"])^(1:20))/20) / 10

b <- par.calibrated
b[names(par.best)] <- par.best

library("igraph")

fname <- paste(proj.home, "pgraph.RData", sep="")
load(fname)


fname <- paste(proj.home, "h13.RData", sep="")
load(fname)
rownames(d.h13) <- as.character(as.numeric(rownames(d.h13)))

prefs <- 1:47

dat.l <- list()
dat.l.1 <- list()
for(p in prefs) {
  p.graph <- p.graph.l[[p]]
  
  fname <- paste(proj.tmp, p, "/i_long_1_m_new.RData", sep="")
  load(fname)
  fname <- paste(proj.tmp, p, "/i_long_1_dist.RData", sep="")
  load(fname)
  colnames(avg.dist.m) <- "dist"
  dat.l[[as.character(p)]] <- cbind(m.dat, avg.dist.m)
  rownames(dat.l[[as.character(p)]]) <- V(p.graph)$name
  dat.l.1[[as.character(p)]] <- cbind(dat.l[[as.character(p)]], merger=as.numeric(V(p.graph)$name))
  rownames(dat.l.1[[as.character(p)]]) <- V(p.graph)$name
  
  qc.dir <- paste(proj.tmp, p, "/", sep="")
  r.l <- list()
  refinements <- system(paste("ls ", qc.dir, "r*_new.RData", sep=""), intern=TRUE)
    for(refinement in refinements) {
      ref.name <- as.numeric(substr(refinement, start=nchar(refinement) - 14, stop=nchar(refinement) - 10))
      r.l[[refinement]] <- new.env()
      load(refinement, envir=r.l[[refinement]])
      r.munis <- r.l[[refinement]]$muni.l[[length(r.l[[refinement]]$muni.l)]]
      r.dat <- r.l[[refinement]]$dat.l[[length(r.l[[refinement]]$dat.l)]]
      dat.l.1[[as.character(p)]][r.munis + 1, colnames(dat.l[[as.character(p)]])] <- r.dat[, colnames(dat.l[[as.character(p)]])]
      dat.l.1[[as.character(p)]][r.munis + 1, "merger"] <- ref.name
  }
}

d0 <- do.call("rbind", dat.l)
stopifnot(d0[,"sfr"] * 100 - d.h13[rownames(d0),"sfr"] < 1)  # rownames had better have lined things up correctly...

d1 <- do.call("rbind", dat.l.1)

fname <- paste(proj.home, "fin_h13.RData", sep="")
load(fname)
rownames(df.h13) <- as.character(as.numeric(rownames(df.h13)))
rownames(num.units) <- as.character(as.numeric(rownames(num.units)))

fname <- paste(proj.home, "hosei.R", sep="")
source(fname)

d.h13.tot <- colSums(d.h13)
d.h13.tot

get.x.v <- function(x.pop, T=0) {
  df.h13.avg <- colSums(df.h13) * x.pop/sum(df.h13[,1])
  idio.avg <- sum(d0[,"sfn.idio"]) * x.pop/sum(df.h13[,1])

  coal.h13 <- df.h13.avg
  coal.h13["density"] <- coal.h13["pop"] / coal.h13["area"] * 100
  coal.num.units <- colSums(num.units) * x.pop/sum(df.h13[,1])
  coal.coef.h13 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H13"]])))
  coal.coef.h20 <- as.vector(eval(calc.coef, envir=c(as.list(coal.h13), hosei[["H20"]])))
  coal.sfn.one <- h13.unit.cost * coal.num.units
  coal.sfn.h13 <- h13.unit.cost * coal.num.units * coal.coef.h13
  coal.sfn.h20 <- h20.unit.cost * coal.num.units * coal.coef.h20

  Y <- (df.h13.avg["sfr"]/100) / (b["tau.bar"] * 3/4)

  c.tilde <- pmax(0, sum(coal.sfn.one)/100 + b["B4"] * (sum(coal.sfn.h13)/100 - sum(coal.sfn.one)/100)) + idio.avg

  q.base <- pmax(0, (b["tau.bar"] * Y + T - b["B3"] * c.tilde)/c.tilde)
  
  ret <- b["B0"] * log((Y - b["B3"]*c.tilde)/Y) + b["B1"] * log((Y - b["B3"]*c.tilde)/c.tilde) - 0.377 * b["B2"] * sqrt(x.pop/340)
  return(ret)
}

optimize(get.x.v, lower=0, upper=1000000, maximum=TRUE)  # about 200000
