# v.R
## returns (structural) payoff for municipalities given characteristics d and parameters b

v.m.new <- function(d, b, time=1, ret="v", T=NA, ninv=FALSE, T0subsidy=TRUE) {
  stopifnot(ret %in% c("v", "tax", "T", "Tpre", "Tminus", "Tpost", "TpostNS"))
  stopifnot(b["B0"] >= 0) # model doesn't make any sense if people don't value private consumption
  stopifnot(b["B1"] >= 0) # ... and the public good
  b0 <- b["B0"]/(b["B0"] + b["B1"])  # expenditure share for private good
  b1 <- b["B1"]/(b["B0"] + b["B1"])  # ... public good
  ## negative idiosyncratic component seems to always be due to large debt repayments, so include with debt
  subsidy <- d[,"sfn.subsidy"] + pmin(0, d[,"sfn.idio"])
  zeta <- pmax(0, d[,"sfn.idio"])

  if(ret=="Tpre") return(pmax(0, d[,"sfn.struct.h13"] + zeta - d[,"sfr"] + subsidy) - subsidy)
  if(ret=="Tpost") return(pmax(0, d[,"sfn.struct.h20"] + zeta - d[,"sfr"] + par.calibrated["disc.tokurei"] * d[,"tokureisai"] + subsidy) - subsidy + par.calibrated["disc.santei.h20"] * d[,"santei.h20"])
  if(ret=="TpostNS") return(pmax(0, d[,"sfn.struct.h20"] + zeta - d[,"sfr"] + subsidy) - subsidy)
  
  if (ninv) {
      c.tilde <- pmax(0, d[,"sfn.struct.h13.ninv"] + zeta)
  } else c.tilde <- pmax(0, d[,"sfn.struct.one"] + b["B4"] * (d[,"sfn.struct.h13"] - d[,"sfn.struct.one"]) + zeta)

  if(is.na(T[1])) {  
      if(time == 0) T <- pmax(0, d[,"sfn.struct.h13"] + zeta - d[,"sfr"] + subsidy) - subsidy + T0subsidy * par.calibrated["disc.santei.h13"] * d[,"santei.h13"]
      if(time == 1) T <- pmax(0, d[,"sfn.struct.h20"] + zeta - d[,"sfr"] + par.calibrated["disc.tokurei"] * d[,"tokureisai"] + subsidy) - subsidy + par.calibrated["disc.santei.h20"] * d[,"santei.h20"]
  }

  Y <- d[,"sfr"] / (b["tau.bar"] * 3/4)

  q.base <- pmax(0, (b["tau.bar"] * Y + T - b["B3"] * c.tilde)/c.tilde)

  Y.net <- pmax(0, Y + T - b["B3"] * c.tilde) # total income net of Stone-Geary expenditure etc.

  q.opt <- b1 * Y.net / c.tilde

  
  use.base <- q.base > q.opt # then provision of public good is inefficiently high
  v.private.base <- rep(b0 * log(1 - b["tau.bar"]), length(Y))
  v.public.base <- b1 * log(q.base)
  if(b["B1"] > 0) {
    v.private.opt <- b0 * log(b0 * Y.net / Y)
    v.public.opt <- b1 * log(q.opt)
  } else {
    v.private.opt <- log(Y.net / Y)
    v.public.opt <- rep(0, length(Y))
  }
  v.public <- v.public.opt
  if(TAX.FLOOR) v.public[use.base] <- v.public.base[use.base]
  v.private <- v.private.opt
  if(TAX.FLOOR) v.private[use.base] <- v.private.base[use.base]
  if(b["B1"] > 0) {
    v <- v.private + v.public
  } else v <- v.private
  if(ret=="v")
    return((b["B0"] + b["B1"]) * v - b["B2"] * d[,"dist"])
  if(ret=="T")
    return(T)
  if(ret=="Tminus")
      return(T - b["B3"] * c.tilde)
  if(ret=="tax") {
    if(b["B1"] > 0) {
      return(((exp(v.public.opt / b1)+b["B3"]) * c.tilde - T) / Y)

    } else {
      return((b["B3"] * c.tilde - T) / Y)
    }
  }
}
